#-------------------------------------------------------------------------#
#
#         Estimation of Tail Risk based on Extreme Expectiles  
#         
#      by Abdelaati Daouia, Stéphane Girard  and Gilles Stupfler 
#                     Updated: June 24, 2017 
#  
#-------------------------------------------------------------------------#

# NOTE: This code is not run time optimized. Problems have not appeared so far, 
#       but their absence cannot be guaranteed for.


#Content: 0) Packages required
#
#         1) Figure 1 in the main paper
#
#         2) Hill's estimator
#
#         3) Weissman quantile estimator
#
#         4.1) True quantile $q_{Y,\tau}$ when (X,Y) has a joint Student distribution on (0,infinity)^2
#         4.2) True expectile $\xi_{Y,\tau}$ when (X,Y) has a joint Student distribution on (0,infinity)^2
#         4.3) True MES=E[X|Y>t] when (X,Y) has a Student t(df) distribution on (0,infty)^2
#         4.4) Comparison QMES vs XMES when (X,Y) has a Student t(df) distribution on (0,infty)^2
#
#         5)  Figure 2 in the main paper
#
#         6) Supplement (A.1) & main paper: 
#
#           6.0.1) Figure 3 in the main paper
#           6.0.2) Figure 4 in the main paper
#           6.1) Tables 1 & 2    
#           6.2) MSE graphs (Figures 1-4)
#           6.3) BIAS graphs (Figures 5-8)
#
#
#         7) Removed from the revision.
#
#         8) Supplement (A.2): 
#
#           8.1) Monte Carlo simulations
#           8.2) Optimal values of "k" 
#           8.3) Figures 9-12
#
#         9) Expectile-based MES:
#
#           9.1) Expectie-based XMES estimator $\widetilde{\textrm{XMES}}^*(\tau'_n)$
#           9.2) QMES estimator $\widehat{\textrm{QMES}}^*(\tau'_n)$
#           9.3) Expectile-based XMES estimator $\widehat{\textrm{XMES}}^*(\tau'_n)$
#           9.4) Monte Carlo experiment
#           9.5) Figures 5-6 in the main paper
#           9.6) QQ-plots in Supplement (A.2)
#           9.7) Figures 13-14 in Supplement (A.2)
#
#        10) Section 8: Applications
#
#          10.1)   Load SOA data
#          10.2)   Histogram of the log claims (Figure 7 top)
#          10.3)   Calculation of the VaR estimates
#          10.4)   Graphs of the estimated VaR (Figure 7)
#          10.5)   Load "daily" and "weekly" data
#          10.6)   Calculations of the estimates of $\gamma_X$, $\gamma_Y$ and QMES and XMES (Supplement)
#          10.7)   Plot of the MES estimates  (Figure 15-a-Supplement)
#          10.8)   Averaged MES estimates for Goldman Sachs
#          10.9)   Plot of the MES estimates  (Figure 15-b-Supplement)
#          10.10)  Averaged MES estimates for Morgan Stanley
#          10.11)  Plot of the MES estimates  (Figure 15-c-Supplement)
#          10.12)  Averaged MES estimates for T. Rowe Price
#          10.13)  Calculations of the estimates of $\gamma_X$, $\gamma_Y$ and QMES and XMES (Main article)
#          10.14)  Plot of the Hill estimates of $\gamma_Y$ and $\gamma_X$ (Figure 8-a-Main article)
#          10.15)  Averaged estimate $\widehat\gamma_{Y}$
#          10.16)  Plot of the MES estimates (Figure 8-b-Main article)
#          10.17)  Averaged MES estimates for Goldman Sachs
#          10.18)  Plot of the MES estimates (Figure 8-c-Main article)
#          10.19)  Averaged MES estimates for Morgan Stanley
#          10.20)  Plot of the MES estimates (Figure 8-d-Main article)
#          10.21)  Averaged MES estimates for T. Rowe Price
#          10.22)  Figure 8 in the main article & Figure 16 in the supplement
#          10.23)  Confidence intervals
#///////////////////////////////////////////////////////////////////////
#///////////////////////////////////////////////////////////////////////


#__________________________________
#
#      (0) Packages required:
#
#__________________________________

##Clear the working space:         
  rm(list = ls())
 

install.packages("expectreg")
require(expectreg)
#
install.packages("ggplot2")
require(ggplot2)
#
library(reshape2)
#
install.packages("mvtnorm")
require(mvtnorm)
#
install.packages("fGarch")
require(fGarch)
#
install.packages("readxl")
require(readxl)




#__________________________________
#
#   (1) Figure 1 in the main paper:
#
#__________________________________

dev.new()

require(grid)
grid.newpage()
pushViewport(viewport(layout = grid.layout(2,1)))

define_region = function(row, col){
	viewport(layout.pos.row=row, layout.pos.col=col)
}


gamma <- seq(0,0.5,length=100)
V1 <- (2* (gamma^3))/(1-(2*gamma));
V2 <- (gamma^2)*(1+((1/(1-gamma))-log((1/gamma)-1))^2);

df <- data.frame(gamma,V1,V2)
names(df) <- c("gamma","V(gamma)","v(gamma)")

ecm <- melt(df, id = "gamma")
dd <- qplot(gamma, value, data=ecm, geom="line", colour=variable)+geom_line(size = 1, linetype=1)
print(dd+scale_color_manual(values = c("V(gamma)" = '#3191ff',"v(gamma)" = '#FA6072')) +
scale_y_continuous("Asymptotic  Variance", limit = c(0,2.5))+
 scale_x_continuous("gamma"),
  vp=define_region(1,1))
 
 
gamma <- seq(0,0.3,length=100)
V1 <- (2* (gamma^3))/(1-(2*gamma));
V2 <- (gamma^2)*(1+((1/(1-gamma))-log((1/gamma)-1))^2);

df <- data.frame(gamma,V1,V2)
names(df) <- c("gamma","V(gamma)","v(gamma)")

ecm <- melt(df, id = "gamma")
dd <- qplot(gamma, value, data=ecm, geom="line", colour=variable)+geom_line(size = 1, linetype=1)
print(dd+scale_color_manual(values = c("V(gamma)" = '#3191ff',"v(gamma)" = '#FA6072')) +
scale_y_continuous("Asymptotic  Variance")+ #, limit = c(0,2.5))+
 scale_x_continuous("gamma")+
labs(title = "Zoom in"),
  vp=define_region(2,1)) 
 
 

#----- Black and white:-------------------------------------------------------
#  Figure 1 :  DGS_variances_fig1.pdf
#-----------------------------------------------------------------------------

dev.new()

require(grid)
grid.newpage()
pushViewport(viewport(layout = grid.layout(1,2)))

define_region = function(row, col){
	viewport(layout.pos.row=row, layout.pos.col=col)
}
#----------------------------
# Left panel :
#----------------------------
gamma <- seq(0,0.5,length=100)
V1 <- (2* (gamma^3))/(1-(2*gamma));
V2 <- (gamma^2)*(1+((1/(1-gamma))-log((1/gamma)-1))^2);

df_V1  <- data.frame(xxc = gamma, yyc = V1)              
df_V2  <- data.frame(xxc = gamma, yyc = V2) 

dd <- ggplot(data=df_V1, aes(xxc, yyc)) + geom_line(size = 0.5, linetype=2) 
print(dd+geom_line(size = .5, data = df_V2, colour = "black", linetype=1) +
scale_y_continuous("Asymptotic  Variance", limit = c(0,2.5))+
#scale_x_continuous(expression(gamma))
scale_x_continuous("gamma")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(1,1))

#----------------------------
# Right panel :
#----------------------------
gamma <- seq(0,0.3,length=100)
V1 <- (2* (gamma^3))/(1-(2*gamma));
V2 <- (gamma^2)*(1+((1/(1-gamma))-log((1/gamma)-1))^2);

df_V1  <- data.frame(xxc = gamma, yyc = V1)              
df_V2  <- data.frame(xxc = gamma, yyc = V2) 

dd <- ggplot(data=df_V1, aes(xxc, yyc)) + geom_line(size = 0.5, linetype=2) 
print(dd+geom_line(size = .5, data = df_V2, colour = "black", linetype=1) +
scale_y_continuous("Asymptotic  Variance")+ 
#scale_x_continuous(expression(gamma))
scale_x_continuous("gamma")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black"))+
labs(title = "Zoom in"),
  vp=define_region(1,2)) 








#__________________________________
#
#       (2) Hill's estimator: 
#
#__________________________________

Hill_inverse  <- function(ytab,k)
{
	  #(Extremiles: by Daouia.A, Juanary 7, 2013)
      #inputs:
      #ytab = sample of nx1 obs
      #    k  = sample fraction
      #output: res = 1/(Hill estimator) = Moment Mn1      
n=length(ytab);
ysort=sort(ytab);   
        g1=ysort[n-k];
        som1=0;
        for (i in 0:(k-1))
        {
            gi=ysort[n-i];
            som1 = som1 + (log(gi)-log(g1));
        }
        res = som1/k;
        return(res);
}




#__________________________________
#
#   (3) Weissman quantile estimator: 
#
#__________________________________

Weissman_quantile <- function(ytab,pn,k)
{
       #(Extremiles: by Daouia.A, Juanary 7, 2013)
       #inputs:
       #       ytab = sample of nx1 obs
       #        pn  = order < 1/n
       #         k  = sample fraction varying from "1" up to "n-1"
       #output: res = Weissman estimator of the extreme (1-pn)th quantile.
 
       n=length(ytab);
       ysort=sort(ytab);   
       gam = Hill_inverse(ytab,k);
       return(ysort[n-k]*(k/(n*pn))^(gam));
}









#____________________________________________________
#
#   (4.1)  True quantile $q_{Y,\tau}$ when (X,Y) has a joint Student distribution on (0,infinity)^2:
#
#____________________________________________________

quantile_transformed_student <- function(df, tau, linf, lsup)
{
       #(by Abdelaati Daouia, March 23, 2016)
       #inputs:
       #         df = degree of freedom of the utilized Student t-distibution
       #        tau = quantile level in (0,1)
       #        [linf,lsup]: domain of optimization 
       #output: true quantile $q_{Y,\tau}$ 

##-------------- Bivariate Student t(df) distribution on (0,infty)^2 : --------------
joint_density <- function(x,y)  ((2/pi)*(1+((x^2+y^2)/df))^(-(df+2)/2))*(x>0)*(y>0)

# marginal density
marginal_Y_density <- function(y)
{ fxy <- function(x)  ((2/pi)*(1+((x^2+y^2)/df))^(-(df+2)/2))*(x>0)*(y>0)
  integrate(fxy, -Inf, Inf)$value
}
##-------------------------------------------------------------------------------- 

density_Y = function(y) unlist(lapply(y, marginal_Y_density))

# Quantile check function
eta <- function(y, tau=0.9) abs(tau-(y<=0))*abs(y)


qu_obj_fct = function(theta, tau)
{ integ_fun <- function(y) eta(y-theta, tau)*density_Y(y)
  integrate(integ_fun, 0, Inf)$value
}

qu_obj_fct_tominimize <- function(theta, tau)
 unlist(lapply(theta, qu_obj_fct, tau=tau))

optimize(qu_obj_fct_tominimize, interval=c(linf,lsup), tau=tau)
}








#____________________________________________________
#
#   (4.2)  True expectile $\xi_{Y,\tau}$ when (X,Y) has a joint Student distribution on (0,infinity)^2:
#
#____________________________________________________

expectile_transformed_student <- function(df, tau, linf, lsup)
{
       #inputs:
       #         df = degree of freedom of the utilized Student t-distibution
       #        tau = expectile level in (0,1)
       #        [linf,lsup]: domain of optimization 
       #output: true expectile $\xi_{Y,\tau}$

## Bivariate Student t(df) distribution on (0,infty)^2 : 
joint_density <- function(x,y)  ((2/pi)*(1+((x^2+y^2)/df))^(-(df+2)/2))*(x>0)*(y>0)

## marginal density
marginal_Y_density <- function(y)
{ fxy <- function(x)  ((2/pi)*(1+((x^2+y^2)/df))^(-(df+2)/2))*(x>0)*(y>0)
  integrate(fxy, -Inf, Inf)$value
}
##
density_Y = function(y) unlist(lapply(y, marginal_Y_density))

# Expectile check function
eta <- function(y, tau=0.9) abs(tau-(y<=0))*y^2

ex_obj_fct = function(theta, tau)
{ integ_fun <- function(y) eta(y-theta, tau)*density_Y(y)
  integrate(integ_fun, 0, Inf)$value
}

ex_obj_fct_tominimize <- function(theta, tau)
 unlist(lapply(theta, ex_obj_fct, tau=tau))

optimize(ex_obj_fct_tominimize, interval=c(linf,lsup), tau=tau)
}




#____________________________________________________
#
#   (4.3)  True MES=E[X|Y>t] when (X,Y) has a Student t(df) distribution on (0,infty)^2:
#
#____________________________________________________

True_MES_student <- function(df,t)
{
       #inputs:
       #         df = degree of freedom of the utilized Student t-distibution
       #          t = point of evaluation 
       #output: E[X|Y>t] when (X,Y) has a Student t(df) distribution on (0,infty)^2


#(1)#-------------- Bivariate Student t(df) distribution on (0,infty)^2 : ---------
joint_density <- function(x,y)  ((2/pi)*(1+((x^2+y^2)/df))^(-(df+2)/2))*(x>0)*(y>0)
##--------------------------------------------------------------------------------- 

#(2)#-------------- Joint prob: P(X>s,Y>t) : --------------
joint <- function(s, t)
{ # [s; Inf] on X and [t;Inf] on Y
 integrate(function(y) { 
   sapply(y, function(y) {  
     integrate(function(x)  joint_density(x,y), s, Inf)$value 
   }) 
  }, t, Inf)$value 
}

##Example (must be 1):
#joint(0,0)
##Example:
#joint(1,1)
##-------------------------------------------------------------------------------- 

#(3)#--------------------- E[X.1(Y>t)}]: ---------------------
double_integ <- function(t)
{ # [t; Inf] on Y and [-Inf;Inf] on X
 integrate(function(y) { 
   sapply(y, function(y) { 
    integrate(function(x)  x*joint_density(x,y), -Inf, Inf)$value 
   }) 
  }, t, Inf)$value 
}
##-------------------------------------------------------------------------------- 

#(4)#--------------------- E[X|Y>t]: ---------------------
true_mse_student <- function(tt)
{
sapply(tt, function(t) double_integ(tt)/joint(-Inf, tt))
}

return(true_mse_student(t))
}







#____________________________________________________
#
#   (4.4)  Comparison QMES vs XMES :
#          when (X,Y) has a Student t(df) distribution on (0,infty)^2
#
#____________________________________________________

      df = 7;

tau  = seq(0.95,0.999,0.001);
nt   = length(tau);
XMES = 0 * c(1:nt);
QMES = 0 * c(1:nt);

for (i in 1:nt)
{
	 #$\xi_{Y,\tau}$:
	 tXVaR = expectile_transformed_student(df, tau[i], linf=0, lsup=50);
     tXVaR = expectile_transformed_student(df, tau[i], linf=(tXVaR$minimum - 2), lsup=(tXVaR$minimum + 2))
     
     #$\textrm{XMES}(\tau)$: 
     XMES[i] = True_MES_student(df,tXVaR$minimum);
 
	 #$q_{Y,\tau}$:
	 tQVaR = quantile_transformed_student(df, tau[i], linf=0, lsup=50);
     tQVaR = quantile_transformed_student(df, tau[i], linf=(tQVaR$minimum - 2), lsup=(tQVaR$minimum + 2))
     
     #$\textrm{QMES}(\tau)$: 
     QMES[i] = True_MES_student(df,tQVaR$minimum); 
}  



#-----------------------------------
#           Save/Load data:
#-----------------------------------
#save(tau,QMES,XMES,file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//t3_MES.RData")
#
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//t3_MES.RData")
#######
#save(tau,QMES,XMES,file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//t5_MES.RData")
#
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//t5_MES.RData")
#######
#save(tau,QMES,XMES,file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//t7_MES.RData")
#
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//t7_MES.RData")
#######
#save(tau,QMES,XMES,file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//t9_MES.RData")
#
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//t9_MES.RData")
#######




#----------------------------------------------------------------------
#
#    (5)  Figure 2 in the main paper :  fig2_dgs_jrssb.pdf
#
#----------------------------------------------------------------------

df <- data.frame(tau,QMES,XMES)             
names(df) <- c("tau","QMES","XMES")

ecm <- melt(df, id = "tau")
dd <- qplot(tau, value, data=ecm, geom="line", colour=variable)+geom_line(size = 3, linetype=1)
dd+scale_color_manual(values = c("QMES" = '#FA8072',"XMES" = '#3399ff')) +
#scale_y_continuous("MES") +
 scale_y_continuous(expression(MES(tau))) +
# scale_x_continuous("tau")+
  scale_x_continuous(expression(tau))+
labs(title = "Student t(9)") #--------------Modify "df" here---------------



#----- Black and white:

dev.new()

require(grid)
grid.newpage()
pushViewport(viewport(layout = grid.layout(1,2)))

define_region = function(row, col){
	viewport(layout.pos.row=row, layout.pos.col=col)
}

#--------------------
#  Left panel : df=3
#--------------------
load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//t3_MES.RData")

df_QMES  <- data.frame(xxc = tau, yyc = QMES)              
df_XMES  <- data.frame(xxc = tau, yyc = XMES) 

dd <- ggplot(data=df_QMES, aes(xxc, yyc)) + geom_line(size = 1, linetype=1) 
print(dd+geom_line(size = 1, data = df_XMES, colour = "black", linetype=2) +
scale_y_continuous("MES") +
#scale_x_continuous(expression(tau))+
scale_x_continuous("tau")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(1,1)) 
  
#--------------------
#  Right panel : df=5
#--------------------
load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//t5_MES.RData")

df_QMES  <- data.frame(xxc = tau, yyc = QMES)              
df_XMES  <- data.frame(xxc = tau, yyc = XMES) 

dd <- ggplot(data=df_QMES, aes(xxc, yyc)) + geom_line(size = 1, linetype=1) 
print(dd+geom_line(size = 1, data = df_XMES, colour = "black", linetype=2) +
scale_y_continuous("MES") +
#scale_x_continuous(expression(tau))+ 
scale_x_continuous("tau")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(1,2)) 











#--------------------------------------------------------------------------------------------
#
#
#
#
#       (6)              Supplement (B.1)  
#                       Bias and MSE estimates
#                        Figures 1-8
#
#                               
#
#
#--------------------------------------------------------------------------------------------
  
##Clear the working space:         
# rm(list = ls())
 
#sample size:-------------------Modify "n" here-------------------------
      n = 100;
      #n = 1000;

#Student's degree of freedom:---------Modify "df" here---------------------
      df = 3;

 
#order "pn < 1/n" [expectile_order = 1-pn]:
    if (n==100){pn = 0.005;}      
    if (n==1000){pn = 0.0006;}                                                      
 
#grid of values of "k":
kl = floor(log(n^0.9))+1;  
     if (n==100){ku = 17;}     
     if (n==1000){ku = floor(n/log(n^0.9));} 
kk = kl:ku; 
nk = length(kk);
 
#Nb of simulated samples:
     if (n==100){B = 10000;}     
     if (n==1000){B = 10000;}


                 #--------------  True expectile VaR: ---------------#
                 #       $\xi_{Y,\tau}$ of Student t(df) distribution
                 #---------------------------------------------------#
                 tVaR = et(1-pn, df) * matrix(1,B,nk);  
                 #---------------------------------------------------#
     

                 # #--------------  True expectile VaR: ---------------#
                 # #    $\xi_{Y,\tau}$ when (X,Y) has a joint Student   
                 # #            distribution on $(0,\infty)^2$ 
                 # #---------------------------------------------------#                
                 # tXVaR = expectile_transformed_student(df, 1-pn, linf=0, lsup=50);
                 # tXVaR = expectile_transformed_student(df,1-pn,linf=(tXVaR$minimum-2),lsup=(tXVaR$minimum+2))                
                 # tVaR  = tXVaR$minimum * matrix(1,B,nk);  
                 # #---------------------------------------------------#


#Initialization of the VaR estimators (B simulated samples & nk estimates for k=kl:ku):
   xp = matrix(0,B,nk);
   xp_als = matrix(0,B,nk);
   

#Monte-Carlo trials:
for (b in 1:B)
{	
                  #--------   simulated data:  ---------#
                  #       Student t(df) distribution
                  #-------------------------------------#    
                  ytab = rt(n, df);       
                  #-------------------------------------#    

             
                  # #--------   simulated data:  ---------#
                  # #    when (X,Y) has a joint Student   
                  # #    distribution on $(0,\infty)^2$ 
                  # #-------------------------------------#
                  # #require(mvtnorm)
                  # Z1_Z2_tab = rmvt(n, sigma = diag(2), df = df, delta = rep(0, 2), type = "shifted");
                  # Z1tab = Z1_Z2_tab[,1];     
                  # Z2tab = Z1_Z2_tab[,2];  
                  # #xtab = abs(Z1tab);      
                  # ytab  = abs(Z2tab);
                  # #-------------------------------------#    

       
                  #computation of the estimates:
                 ysort=sort(ytab); 
                 for (i in 1:nk)
                 {        
            	   k=kk[i];
               	 ##Hill's EVI:
            	   gam = Hill_inverse(ytab,k);
        	     ##Inverse of Hill's EVI:
        	       rho = 1/gam;
                  ##quantile-VaR $\hat{q}_{1-p_n}^{\star}$: 
                   qu = Weissman_quantile(ytab,pn,k);        
                  ##Indirect expectile-VaR $\hat\xi_{1-p_n}^{\star}$:
                   xp[b,i] = (rho - 1)^(-gam) * qu;
                  ##Direct expectile-VaR $\widetilde\xi_{1-p_n}^{\star}$:
                   xp_als[b,i] = (n*pn/k)^(-gam) * expectile(ytab,probs=1-(k/n)); #use the "expectreg" package
                 }
}

#---------------------------------------------------------------------------------------------------
#                              MSE & Bias results:                                          
#---------------------------------------------------------------------------------------------------

        xp_MSE = colMeans((tVaR - xp)^2,na.rm=TRUE); #Indirect (plug-in) Xpectile estimator
        xp_als_MSE = colMeans((tVaR - xp_als)^2);    #Direct (LAWS) Xpectile estimator
        
        xp_Bias = colMeans(xp - tVaR,na.rm=TRUE);      
        xp_als_Bias = colMeans(xp_als - tVaR);
       
        
       #  #-------------------------
       #  # In the case n=1000:
       #  #-------------------------
       #  kk = kk[2:154];
       #  xp_als_MSE = sqrt(xp_als_MSE[2:154]);
       #  xp_MSE = sqrt(xp_MSE[2:154]);  
       #  xp_als_Bias = xp_als_Bias[2:154];
       #  xp_Bias = xp_Bias[2:154];
        
             
       
         #-------------------------
         # In the case n=100:
         #-------------------------
         xp_als_MSE = sqrt(xp_als_MSE);
         xp_MSE = sqrt(xp_MSE);       


       
        ###########################################
        #          Normalized Root-MSE            #
        ###########################################        
        xp_als_NMSE = xp_als_MSE/abs(tVaR[1,1]);
        xp_NMSE = xp_MSE/abs(tVaR[1,1]);
                   
        ###########################################
        #             Normalized BIAS             #
        ###########################################        
        xp_NBias = xp_Bias/tVaR[1,1];
        xp_als_NBias = xp_als_Bias/tVaR[1,1];


#---------------------------------------------------------------------------------------------------
#Save data:
#---------------------------------------------------------------------------------------------------

#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,
file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//student3_n100.RData")
##
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,
#file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//student5_n100.RData")
##
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,
#file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//student7_n100.RData")
##
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,
#file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//student9_n100.RData")
##
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,
file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//student9_n1000.RData")
##
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,
#file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//student7_n1000.RData")
##
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,
#file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//student5_n1000.RData")
##
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,
#file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//student3_n1000.RData")





## n = 1000 :
##
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,
#file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//studentt3_n1000.RData")
##
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,
#file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//studentt5_n1000.RData")
##
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,
#file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//studentt7_n1000.RData")
##
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,
#file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//studentt9_n1000.RData")
##
## n = 100 :
##
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,
#file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//studentt9_n100.RData")
##
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,
file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//studentt7_n100.RData")
##
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,
#file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//studentt5_n100.RData")
##
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,
#file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//studentt3_n100.RData")
##



#---------------------------------------------------------------------------------------------------
#Load data:
#---------------------------------------------------------------------------------------------------

#### Student: 
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//student3_n100.RData")
##
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//student5_n100.RData")
##
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//student7_n100.RData")
##
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//student9_n100.RData")
##
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//student9_n1000.RData")
##
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//student7_n1000.RData")
##
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//student5_n1000.RData")
##
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//student3_n1000.RData")


#### Truncated Student:
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//studentt3_n1000.RData")
##
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//studentt5_n1000.RData")
##
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//studentt7_n1000.RData")
##
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//studentt9_n1000.RData")
##
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//studentt3_n100.RData")
##
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//studentt5_n100.RData")
##
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//studentt7_n100.RData")
##
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//studentt9_n100.RData")
##


#---------------------------------------------------------------------------------------------------
#
#
#    (6.0.1)  Figure 3 in the main paper:  fig3_dgs_jrssb.pdf
#
#                             
#---------------------------------------------------------------------------------------------------
dev.new()

require(grid)
grid.newpage()
pushViewport(viewport(layout = grid.layout(2,2)))

define_region = function(row, col){
	viewport(layout.pos.row=row, layout.pos.col=col)
}

#-------------
# RMSE df=3
#-------------
load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//student3_n1000.RData")

df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NMSE)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NMSE) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = 1, linetype=1) 
print(dd+geom_line(size = 1, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("RMSE") +
scale_x_continuous("k")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(1,1))


#-------------
# RMSE df=5
#-------------
load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//student5_n1000.RData")

df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NMSE)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NMSE) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = 1, linetype=1) 
print(dd+geom_line(size = 1, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("RMSE") +
scale_x_continuous("k")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(1,2))


#-------------
# Bias df=3
#-------------
load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//student3_n1000.RData")

df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NBias)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NBias) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = 1, linetype=1) 
print(dd+geom_line(size = 1, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("BIAS") +
scale_x_continuous("k")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(2,1))


#-------------
# Bias df=5
#-------------
load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//student5_n1000.RData")

df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NBias)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NBias) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = 1, linetype=1) 
print(dd+geom_line(size = 1, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("BIAS") +
scale_x_continuous("k")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(2,2))






#---------------------------------------------------------------------------------------------------
#
#
#    (6.0.2)  Figure 4 in the main paper:  fig4_dgs_jrssb.pdf
#
#                             
#---------------------------------------------------------------------------------------------------
dev.new()

require(grid)
grid.newpage()
pushViewport(viewport(layout = grid.layout(2,2)))

define_region = function(row, col){
	viewport(layout.pos.row=row, layout.pos.col=col)
}

#-------------
# RMSE df=3
#-------------
load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//studentt3_n1000.RData")

df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NMSE)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NMSE) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = 1, linetype=1) 
print(dd+geom_line(size = 1, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("RMSE") +
scale_x_continuous("k")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(1,1))


#-------------
# RMSE df=5
#-------------
load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//studentt5_n1000.RData")

df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NMSE)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NMSE) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = 1, linetype=1) 
print(dd+geom_line(size = 1, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("RMSE") +
scale_x_continuous("k")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(1,2))


#-------------
# Bias df=3
#-------------
load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//studentt3_n1000.RData")

df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NBias)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NBias) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = 1, linetype=1) 
print(dd+geom_line(size = 1, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("BIAS") +
scale_x_continuous("k")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(2,1))


#-------------
# Bias df=5
#-------------
load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//studentt5_n1000.RData")

df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NBias)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NBias) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = 1, linetype=1) 
print(dd+geom_line(size = 1, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("BIAS") +
scale_x_continuous("k")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(2,2))






#---------------------------------------------------------------------------------------------------
#
#
#    (6.1)  Tables 1 & 2 (Supplement)
#
#
#---------------------------------------------------------------------------------------------------             
         ii_xp_als =which (xp_als_NMSE==min(xp_als_NMSE));
         xp_als_NMSE_min =  xp_als_NMSE[ii_xp_als];
         xp_als_NBIAS_min =  xp_als_NBias[ii_xp_als];
         #
         ii_xp =which (xp_NMSE==min(xp_NMSE));
         xp_NMSE_min =  xp_NMSE[ii_xp];
         xp_NBIAS_min =  xp_NBias[ii_xp];
         #
         #----------------------------------------------------------
         c(xp_als_NMSE_min, xp_NMSE_min, xp_als_NBIAS_min, xp_NBIAS_min)
         #----------------------------------------------------------







#---------------------------------------------------------------------------------------------------
#
#
#    (6.2)  MSE graphs (Figures 1-4): 
#
#                             
#---------------------------------------------------------------------------------------------------


df2_NMSE <- data.frame(kk,xp_als_NMSE,xp_NMSE)              
names(df2_NMSE) <- c("Sample_fraction_k","LAWS","Plug-in")

ecm <- melt(df2_NMSE, id = "Sample_fraction_k")

dd <- qplot(Sample_fraction_k, value, data=ecm, geom="line", colour=variable)+geom_line(size = 4, linetype=1)
dd+scale_color_manual(values = c("LAWS" = '#3399ff',"Plug-in" = '#FA8072')) +
scale_y_continuous("root  MSE") +
scale_x_continuous("Sample  fraction  k")+
#labs(title = "Student t(3) , n=100") #--------------Modify "df" & "n" here---------------
labs(title = "Truncated  Student t(9) , n=1000") #--------------Modify "df" & "n" here---------------



#----- Black and white:


df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NMSE)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NMSE) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = 1.5, linetype=1) 
dd+geom_line(size = 1.5, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("RMSE") +
scale_x_continuous("k")





#---------------------------------------------------------------------------------------------------
#
#
#    (6.3)  BIAS graphs (Figures 5-8): 
#
#                             
#---------------------------------------------------------------------------------------------------


df2_NBIAS <- data.frame(kk,xp_als_NBias,xp_NBias)     
names(df2_NBIAS) <- c("Sample_fraction_k","LAWS","Plug-in")

ecm <- melt(df2_NBIAS, id = "Sample_fraction_k")

dd <- qplot(Sample_fraction_k, value, data=ecm, geom="line", colour=variable)+geom_line(size = 4, linetype=2)
dd+scale_color_manual(values = c("LAWS" = 'blue',"Plug-in" = 'red')) +
scale_y_continuous("BIAS") +
scale_x_continuous("Sample  fraction  k")+
#labs(title = "Student t(9) , n=1000") #--------------Modify "df" & "n" here---------------
labs(title = "Truncated  Student t(9) , n=1000") #--------------Modify "df" & "n" here---------------



#----- Black and white:


df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NBias)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NBias) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = 2, linetype=1) 
dd+geom_line(size = 2, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("BIAS") +
scale_x_continuous("k")







#--------------------------------------------------------------------------------------------
#
#
#
#
#              (7)      Supplement (B.2) : 
#                       Forecast verification and validation,
#                       Figures 9-11.
#
#                               
#
#
#--------------------------------------------------------------------------------------------
  

#__________________________________
#
#   (7.1) Consistent Loss function: 
#
#__________________________________

eta <- function(tau, y)
{
       #output:  eta_{tau}(y)
       return(abs(tau-(y <= 0)) * y^2);
}



Loss <- function(tau, xi, r)
{
       #output:  L_{\tau}(xi,r) = eta_{tau}(r-xi)
       return(eta(tau,r-xi));
}




eta_quant <- function(tau, y)
{
       #output:  eta_{tau}(y)
       return(abs(tau-(y <= 0)) * abs(y));
}



Loss_quant <- function(tau, xi, r)
{
       #output:  L_{\tau}(xi,r) = rho_{tau}(r-xi)
       return(eta_quant(tau,r-xi));
}




#____________________________________________________
#
#   (7.2) Monte Carlo simulations:
#
#____________________________________________________
  

#size of the global time series :
  N = 1500;
#size of the rolling windows :  
  n = 1000;
#Number of forecast cases :
  T = N-n;
  
 
  
#Student's degree of freedom:
#-------------
   df = 9;     #--------------------------Modify "df" here---------------------
#-------------
 
 
#order "pn < 1/n" [expectile_order = 1-pn]:
    if (n==100){pn = 0.005;}      
    if (n==1000){pn = 0.0006;}    
                                                
 
#grid of values of "k":
kl = floor(log(n^0.9))+1;  
     if (n==100){ku = 17;}     
     if (n==1000){ku = floor(n/log(n^0.9));} 
kk = kl:ku; 
nk = length(kk);
 
#Nb of simulated samples:
     if (n==100){B = 500000;}      
     if (n==1000){B = 10000;} #{B = 200;}
 

#Initialization of the sum of loss functions:
##B-->simulated sample & nk-->estimates for k=kl:ku
SLtilde =  0* c(1:nk); 
SLhat   =  0* c(1:nk); 


#Monte-Carlo trials:
for (b in 1:B)
{
	
                  # #-- Simulated global time series: ----#
                  # #     (1) Student t(df) distribution
                  # #-------------------------------------#    
                  # ytab = rt(N, df);       
                  # #-------------------------------------#    

          
                   #-- Simulated global time series: ----#
                   #   (2) when (X,Y) has a joint Student   
                   #    distribution on $(0,\infty)^2$ 
                   #-------------------------------------#
                   #require(mvtnorm)
                   Z1_Z2_tab = rmvt(N, sigma = diag(2), df = df, delta = rep(0, 2), type = "shifted");
                   Z1tab = Z1_Z2_tab[,1];     
                   Z2tab = Z1_Z2_tab[,2];  
                   #xtab = abs(Z1tab);      
                   ytab  = abs(Z2tab);
                   #-------------------------------------#    
 
    
                 # #-- Simulated global time series: ----#
                 # # (3) sstd-GARCH(1,1) ---- using defaults with skew student:
                 #  model = garchSpec(model = list(skew = 0.93, shape = df), cond.dist = "sstd")
                 #  ytab =  garchSim(model, n = N)$garch;                                 
 
         
       #Initialization of loss functions:          
       Ltilda   = matrix(0,T,nk);
       Lchapeau = matrix(0,T,nk);
                
       for (t in 1:T)
               {                         
                 #Rolling window sample:
                 yytab = ytab[t:(n+t-1)]; 
                 
                 #Realizing observation "r_t": 
                 realobs = ytab[n+t];  
                 
              for (i in 1:nk)
               {        
 
            	 k=kk[i];
               	 ##Hill's EVI:
            	 gam = Hill_inverse(yytab,k);
        	         ##Inverse of Hill's EVI:
        	         rho= 1/gam;
        	         
                  ##quantile-VaR $\hat{q}_{1-p_n}^{\star}$: 
                  quweissman = Weissman_quantile(yytab,pn,k);                          
                  ##Indirect expectile-VaR $\hat\xi_{1-p_n}^{\star}$:
                  xp_plug = (rho - 1)^(-gam) * quweissman;
 
                  ##Direct expectile-VaR $\widetilde\xi_{1-p_n}^{\star}$:
                  xp_laws = (n*pn/k)^(-gam) * expectile(yytab,probs=1-(k/n)); #use the "expectreg" package                 
                  
                  #Loss functions "here: \tau_n' = 1-pn":
                  Ltilda[t,i]   = Loss(1-pn, xp_laws, realobs);
                  Lchapeau[t,i] = Loss(1-pn, xp_plug, realobs);  
                 } #fin de la boucle sur i=1:nk                 
           } #fin de la boucle sur t=1:T
   
  #Realized losses:               
  Ltilde = colMeans(Ltilda);    #Ltilde = colMeans(Ltilda,na.rm=TRUE);
  Lhat   = colMeans(Lchapeau);    

  SLtilde = SLtilde + Ltilde;
  SLhat   = SLhat   + Lhat; 
} #fin de la boucle sur b=1:B

#Averaged losses (functions of kk):
  ALtilde = SLtilde/B;
  ALhat   = SLhat/B; 


        kk      = kk[1:134];
        ALtilde = ALtilde[1:134];
        ALhat   = ALhat[1:134];


###########
#Save data:
###########
#save(kk,  ALtilde,  ALhat, file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//forecast_t3.RData")
#save(kk,  ALtilde,  ALhat, file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//forecast_t5.RData")
#save(kk,  ALtilde,  ALhat, file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//forecast_t7.RData")
#save(kk,  ALtilde,  ALhat, file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//forecast_t9.RData")
#
#save(kk,  ALtilde,  ALhat, file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//garch_t3.RData")
#save(kk,  ALtilde,  ALhat, file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//garch_t5.RData")
#save(kk,  ALtilde,  ALhat, file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//garch_t7.RData")
#save(kk,  ALtilde,  ALhat, file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//garch_t9.RData")
#
#save(kk,  ALtilde,  ALhat, file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//forecast_tt3.RData")
#save(kk,  ALtilde,  ALhat, file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//forecast_tt5.RData")
#save(kk,  ALtilde,  ALhat, file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//forecast_tt7.RData")
#save(kk,  ALtilde,  ALhat, file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//forecast_tt9.RData")



###########
#Load data:
###########
#### Student:
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//forecast_t3.RData")
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//forecast_t5.RData")
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//forecast_t7.RData")
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//forecast_t9.RData")

#### truncated Student:
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//forecast_tt3.RData")
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//forecast_tt5.RData")
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//forecast_tt7.RData")
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//forecast_tt9.RData")

#### sstd-GARCH:
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//garch_t3.RData")
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//garch_t5.RData")
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//garch_t7.RData")
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//garch_t9.RData")





#____________________________________________________
#
#   (7.3) Figures 9, 10 and 11:
#
#____________________________________________________
  

df <- data.frame(kk,  ALtilde,  ALhat)     
names(df) <- c("Sample_fraction_k","LAWS","Plug-in")

ecm <- melt(df, id = "Sample_fraction_k")
dd <- qplot(Sample_fraction_k, value, data=ecm, geom="line", colour=variable)+geom_line(size = 1, linetype=1)
dd+scale_color_manual(values = c("LAWS" = '#3399ff',"Plug-in" = 'red')) +
scale_y_continuous("Realized  losses") +
#scale_y_continuous("Realized  losses", limit = c(0.25,0.42)) +
scale_x_continuous("Sample  fraction  k")+
#--------------------------------
labs(title = "Student t(3)")  
#labs(title = "Truncated  Student t(9)")     #--------------Modify "df" here---------------
#labs(title = "sstd-GARCH(1,1) -- shape=3")     
#--------------------------------




#----- Black and white:


df_xp_als  <- data.frame(xxc = kk, yyc = ALtilde)              
    df_xp  <- data.frame(xxc = kk, yyc = ALhat) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = .75, linetype=1) 
dd+geom_line(size = 1.3, data = df_xp, colour = "black", linetype=2) +
#
scale_y_continuous("Realized  Losses") +
#scale_y_continuous("Realized  Losses", limit = c(0.25,0.42)) +
scale_x_continuous("Sample  fraction  k")









#--------------------------------------------------------------------------------------------
#
#
#
#
#              (8)      Supplement (B.3) : 
#                       Quality of asymptotic approximations,
#                       Figures 12-15.
#
#                               
#
#
#--------------------------------------------------------------------------------------------
  

#__________________________________
#
#   (8.1) Monte Carlo simulations:
#
#__________________________________

 
#sample size:
      n = 1000;
      
      
#Student's degree of freedom:---------Modify "df" here---------------------
df = 9;

 
#order "pn < 1/n" [expectile_order = 1-pn]:
    if (n==1000){pn = 0.0006;}    
       
                                         
 
#grid of values of "k":
kl = floor(log(n^0.9))+1;       
     if (n==1000){ku = floor(n/log(n^0.9));} 
kk = (kl+2):ku; 
nk = length(kk);
 
#Nb of simulated samples:
     if (n==1000){B = 10000;}     
     #if (n==1000){B = 2000;}
         

                 # #--------------  True expectile VaR: ---------------#
                 # #       $\xi_{Y,\tau}$ of Student t(df) distribution
                 # #---------------------------------------------------#
                 # tVaR = et(1-pn, df) * matrix(1,B,nk);  
                 # #---------------------------------------------------#
    

                  #--------------  True expectile VaR: ---------------#
                  #    $\xi_{Y,\tau}$ when (X,Y) has a joint Student   
                  #            distribution on $(0,\infty)^2$ 
                  #---------------------------------------------------#                
                  tXVaR = expectile_transformed_student(df, 1-pn, linf=0, lsup=50);
                  tXVaR = expectile_transformed_student(df,1-pn,linf=(tXVaR$minimum-2),lsup=(tXVaR$minimum+2))                
                  tVaR  = tXVaR$minimum * matrix(1,B,nk);  
                  #---------------------------------------------------#


#************************ Initialization of the VaR & ES estimators: **************************
##B-->simulated sample & nk-->estimates for k=kl:ku

 xp         = matrix(0,B,nk);
 xp_als     = matrix(0,B,nk);

xp_laws_opt = 0 * c(1:B);
xp_plug_opt = 0 * c(1:B);

Wntilde     = 0 * c(1:B);
Wnhat       = 0 * c(1:B);

kopt_laws   =  0 * c(1:B);
kopt_plug   =  0 * c(1:B);


#Monte-Carlo trials:
for (b in 1:B)
{ 	
                  # #--------   simulated data:  ---------#
                  # #       Student t(df) distribution
                  # #-------------------------------------#    
                  # ytab = rt(n, df);       
                  # #-------------------------------------#    
                                       
             
                    #--------   simulated data:  ---------#
                    #    when (X,Y) has a joint Student   
                    #    distribution on $(0,\infty)^2$ 
                    #-------------------------------------#
                    #require(mvtnorm)
                    Z1_Z2_tab = rmvt(n, sigma = diag(2), df = df, delta = rep(0, 2), type = "shifted");
                    Z1tab = Z1_Z2_tab[,1];     
                    Z2tab = Z1_Z2_tab[,2];  
                    #xtab = abs(Z1tab);      
                    ytab  = abs(Z2tab);
                    #-------------------------------------#    

                                         
                 #computation of the estimates:
                 ysort=sort(ytab); 
                 
           for (i in 1:nk)
              {        
            	 k=kk[i];
               	 ##Hill's EVI:
            	 gam = Hill_inverse(ytab,k);
            	 #gam = 1/df;
        	     ##Inverse of Hill's EVI:
        	         rho= 1/gam;
                  ##quantile-VaR $\hat{q}_{1-p_n}^{\star}$: 
                   qubi=Weissman_quantile(ytab,pn,k);   
                     
                  ##Indirect expectile-VaR
                   xp[b,i] = (rho - 1)^(-gam) * qubi;
                                     
                  ##Direct expectile-VaR $\widetilde\xi_{1-p_n}^{\star}$:
                   xp_als[b,i] = (n*pn/k)^(-gam) * expectile(ytab,probs=1-(k/n)); #use the "expectreg" package
                 }                 
}



#---------------------------------------------------------------------------------------------------
#                                         Save data:  
#---------------------------------------------------------------------------------------------------

#save(kk,  xp_als,  xp, file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//t3_n1000.RData")
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//t3_n1000.RData")



#---------------------------------------------------------------------------------------------------
####                         The optimal "k" minimizing the MSE:                                ####
#---------------------------------------------------------------------------------------------------

xp_als_MSE = colMeans((tVaR - xp_als)^2);    #Direct (LAWS) Xpectile estimator
         #
         ii_xp_als   = which (xp_als_MSE==min(xp_als_MSE));
         k_tilde        = kk[ii_xp_als];
                          
         
xp_MSE = colMeans((tVaR - xp)^2,na.rm=TRUE); #Indirect (plug-in) Xpectile estimator
         #
         ii_xp       = which (xp_MSE==min(xp_MSE));
         k_hat         = kk[ii_xp];         
 
         
# k_hat

# k_tilde

##########################$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

A_fct <- function(s,df)
    {
       n = df;
       num = 2*gamma((n+1)/2) * n^((n-1)/2);
       den = sqrt(n*pi) * gamma(n/2);      
       #c_n=\frac{2 \Gamma((n+1)/2) n^{(n-1)/2}}{\sqrt{n\pi} \Gamma(n/2)} :
       c_n = num/den;
       #A(s)\sim \frac{n+1}{n+2} (c_n s)^{-2/n} :
       res = ((n+1)/(n+2)) * (c_n * s)^(-2/n);
       return(res);
     }
     

#Monte-Carlo trials:
for (b in 1:B)
{                                   
                  # #--------   simulated data:  ---------#
                  # #       Student t(df) distribution
                  # #-------------------------------------#    
                  # ytab = rt(n, df);       
                  # #-------------------------------------#    
                                      
             
                   #--------   simulated data:  ---------#
                   #    when (X,Y) has a joint Student   
                   #    distribution on $(0,\infty)^2$ 
                   #-------------------------------------#
                   #require(mvtnorm)
                   Z1_Z2_tab = rmvt(n, sigma = diag(2), df = df, delta = rep(0, 2), type = "shifted");
                   Z1tab = Z1_Z2_tab[,1];     
                   Z2tab = Z1_Z2_tab[,2];  
                   #xtab = abs(Z1tab);      
                   ytab  = abs(Z2tab);
                   #-------------------------------------#    
                                             
                 #computation of the estimates:
                 ysort=sort(ytab); 
                 
                       	
               	 ##Hill's EVI:
            	 gamhat = Hill_inverse(ytab,k_hat);
            	 gamtilde = Hill_inverse(ytab,k_tilde);

        	     
        	     ##Inverse of Hill's EVI:
        	         rhohat= 1/gamhat;
         	         rhotilde= 1/gamtilde;
 
                  ##quantile-VaR $\hat{q}_{1-p_n}^{\star}$: 
                   qubi=Weissman_quantile(ytab,pn,k_hat);   
                     
        ##Indirect expectile-VaR
        xp_plug_opt = (rhohat - 1)^(-gamhat) * qubi;
                                     
        ##Direct expectile-VaR $\widetilde\xi_{1-p_n}^{\star}$:
        xp_laws_opt = (n*pn/k_tilde)^(-gamtilde)*expectile(ytab,probs=1-(k_tilde/n)); #use `expectreg' package                
                     
         truexp      = tVaR[1,1];  
         gama        = 1/df;
         rrho        = -2/df;  
                             
         lambda2t    =  sqrt(k_tilde) * A_fct((n/k_tilde),df);      
         rnt         =  sqrt(k_tilde)/(log( k_tilde/(n*pn) ));          
         #Wntilde[b] = ( (rnt * ((xp_laws_opt/truexp)-1))  - (lambda2t/(1-rrho)) )/gama;     
         Wntilde[b]  = ( (rnt * log(xp_laws_opt/truexp))  - (lambda2t/(1-rrho)) )/gama;   
 
         lambda2h    =  sqrt(k_hat) * A_fct((n/k_hat),df);      
         rnh         =  sqrt(k_hat)/(log( k_hat/(n*pn) ));          
         #Wntilde[b] = ( (rnh * ((xp_plug_opt/truexp)-1))  - (lambda2h/(1-rrho)) )/gama;   
         Wnhat[b]    = ( (rnh * log(xp_plug_opt/truexp))  - (lambda2h/(1-rrho)) )/gama;                    
}

##########################$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$



#____________________________________________________
#
#   (8.2) Optimal values of "k":
#
#____________________________________________________


k_hat

k_tilde


#____________________________________________________
#
#   (8.3) Figures 12-15:
#
#____________________________________________________

#qqnorm(Wntilde, main="LAWS", xlab="Theoretical Quantiles", ylab="Sample Quantiles", col=4)
#qqline(Wntilde, col = 1, lwd = 2)
#
#
#qqnorm(Wnhat, main="Plug-in", xlab="Theoretical Quantiles", ylab="Sample Quantiles", col=2)
#qqline(Wnhat, col = 1, lwd = 2)


qplot(sample = Wnhat, stat="qq")+
labs(title="t(9) , Plug-in , k=15")


qplot(sample = Wntilde, stat="qq")+
labs(title="t(9) , LAWS , k= 11")


#---------------------------------------------------------------------------------------------------





  
  
  
 
#--------------------------------------------------------------------------------------------
#
#
#
#
#              (9)      Expectile-based MES: 
#                             
#                           Figures 5-6 in the main paper.
#                           Figures 16-17 in Supplement (A.2)
#                               
#
#
#--------------------------------------------------------------------------------------------
  
 
#__________________________________
#
#   (9.1)  Expectie-based XMES estimator $\widetilde{\textrm{XMES}}^*(\tau'_n)$: 
#
#__________________________________

XMESt <- function(xtab,ytab,pn,k)
{
       #inputs:
       #       xtab = sample of nx1 obs
       #       ytab = sample of nx1 obs
       #        pn  = order < 1/n
       #         k  = sample fraction varying from "1" up to "n-1"
       #output: XMES estimator $\widetilde{\textrm{XMES}}^*(\tau'_n)$
 
       #sample size:
       n=length(ytab);  
       
       #$\widetilde{\xi}_{Y,\tau_n}$ with $\tau_n = 1-k/n$: 
       xi_Y = expectile(ytab,probs=1-(k/n)); #use the "expectreg" package
       
       #$\widetilde{\textrm{XMES}}(\tau_n)$ with $\tau_n = 1-k/n$:
       num = sum(xtab*(ytab > xi_Y));      
       den = sum(ytab > xi_Y);       
       XMES = num/den;
       
       #$\widehat{\gamma}_X$:
       gamX = Hill_inverse(xtab,k);
                gamX = min(gamX,1/2);
                       
       #$\widetilde{\textrm{XMES}}^*(\tau'_n)$ with $\tau'_n = 1-pn$:
       res = (n*pn/k)^(-gamX) * XMES;       
       return(res);
}



#__________________________________
#
#   (9.2) QMES estimator $\widehat{\textrm{QMES}}^*(\tau'_n)$: 
#
#__________________________________

QMESh <- function(xtab,ytab,pn,k)
{
       #inputs:
       #       xtab = sample of nx1 obs
       #       ytab = sample of nx1 obs
       #        pn  = order < 1/n
       #         k  = sample fraction varying from "1" up to "n-1"
       #output: QMES estimator $\widehat{\textrm{QMES}}^*(\tau'_n)$
 
       #sample size:
       n=length(ytab);         
       
       #intermediate quantile $\widehat{q}_{Y,\tau_n}:=Y_{n-\lfloor n(1-\tau_n) \rfloor,n}$: 
       ysort = sort(ytab);
       q_Y = ysort[n-k]; 
       
       #$\widehat{\textrm{QMES}}(\tau_n)$ with $\tau_n = 1-k/n$:
       num = sum(xtab*(ytab > q_Y));      
       den = k;       
       QMES = num/den;
              
       #$\widehat{\gamma}_X$:
       gamX = Hill_inverse(xtab,k);
                gamX = min(gamX,1/2);
                       
       #$\widehat{\textrm{QMES}}^*(\tau'_n)$ with $\tau'_n = 1-pn$:
       res = ((n*pn)/k)^(-gamX) * QMES;       
       return(res);
}




#__________________________________
#
#   (9.3)  Expectile-based XMES estimator $\widehat{\textrm{XMES}}^*(\tau'_n)$: 
#
#__________________________________

XMESh <- function(xtab,ytab,pn,k)
{
       #(Extremiles: by Daouia.A, March, 2016)
       #inputs:
       #       xtab = sample of nx1 obs
       #       ytab = sample of nx1 obs
       #        pn  = order < 1/n
       #         k  = sample fraction varying from "1" up to "n-1"
       #output: XMES estimator $\widehat{\textrm{XMES}}^*(\tau'_n)$
 
gamX = Hill_inverse(xtab,k);
                gamX = min(gamX,1/2);
gamY = Hill_inverse(ytab,k); 
                gamY = min(gamY,1/2);
 
res = ((1/gamY) - 1)^(-gamX) * QMESh(xtab,ytab,pn,k);
return(res);
}




#____________________________________________________
#
#   (9.4) Monte Carlo experiment :
#
#____________________________________________________
   
#sample size:
n = 1000;

#order "pn < 1/n" [expectile_order = 1-pn]:   
    if (n==1000){pn = 0.0006;}    
                                         
#grid of values of "k":
kl = floor(log(n^0.9))+1;       
     if (n==1000){ku = floor(n/log(n^0.9));} 
kk = (kl+2):ku; 
nk = length(kk);
 
#Nb of simulated samples:     
     if (n==1000){B = 10000;}



# CHOOSE a degree of freedom :
  
         df = 9;  



##****************** (1) True QMES(alpha_n) = XMES(\tau_n'(\alpha_n)): ***************** 
#                   #    when (X,Y) has a joint Student t(df)  
#                   #     distribution on $(0,\infty)^2$ : 
#  
#  
#  tQVaR = quantile_transformed_student(df=df, 1-pn, linf=0, lsup=50);
#  tQVaR = quantile_transformed_student(df=df, 1-pn, linf=(tQVaR$minimum - 2), lsup=(tQVaR$minimum + 2));
#  
#  tQMES = True_MES_student(df=df, tQVaR$minimum);
#  tXMES = tQMES * matrix(1,B,nk);  
#
##************************  End True MES Student  ************************************ 



##****************** (2) True QMES(alpha_n) = XMES(\tau_n'(\alpha_n)): ***************** 
#                   #    when (X,Y) has a positive cauchy 
#                   #     distribution on $(0,\infty)^2$ : 
#     
#theta_p=function(p,gamma1)
#{U2p=tan((1-p)*pi/2); #U_2(1/p)
#intgrand=function(x){x^(gamma1)/(1+x^2)*(1-U2p/sqrt(1+x^2+U2p^2));}
#result=2/p/pi*integrate(intgrand,0,Inf,abs.tol=5e-30)$value;  # real theta_p
#return(result);
#}
#    
#  tQMES = theta_p(pn, 2/5) 
#  tXMES = tQMES * matrix(1,B,nk); 
#
##************************  End True MES  positive cauchy  ***************************** 



##****************** (3) True QMES(alpha_n) = XMES(\tau_n'(\alpha_n)): ***************** 
#                   #    when (X,Y) has a general cauchy 
#                   #     distribution on $\R^2$ : 
#     
#theta_p=function(p,gamma1)
#{U2p=uniroot(function(y){1/2-(atan(y)+atan(y^3))/2/pi-p},c(0,1/p))$root;
#intgrand=function(x)
#{x^(gamma1)/(1+x^2)*(1-U2p/sqrt(1+x^2+U2p^2))-x^(gamma1/2)/(1+x^2)*(1-U2p^3/sqrt(1+x^2+U2p^6));}
#result=1/(2*p*pi)*integrate(intgrand,0,Inf,abs.tol=5e-30)$value;
#return(result);
#}
#    
#  tQMES = theta_p(pn, 2/5) 
#  tXMES = tQMES * matrix(1,B,nk); 
#
##************************  End True MES  general cauchy  ***************************** 



#****************** (4) True QMES(alpha_n) = XMES(\tau_n'(\alpha_n)): ***************** 
                   #    when (X,Y) has a Transformed Student t(df) 
                   #     distribution on $\R^2$ : 
       
 theta_p=function(p,df){ 
  U2p=qt(1-p,df);
  gamma1=df/4;
  intgrand=function(x){
    (gamma((df+1)/2)/(sqrt(df*pi)*gamma(df/2)))*(1+(x^2)/df)^(-(df+1)/2)*(1-pt(U2p*sqrt(df+1)/sqrt(x^2+df),df+1))*(x^(gamma1)-x^(gamma1/2))    
    }
  result=(1/p)*integrate(intgrand,0,Inf,abs.tol=5e-30)$value;
  return(result);
}

 
  tQMES = theta_p(pn, df)
  tXMES = tQMES * matrix(1,B,nk); 

#*********************** End True MES  Transformed Student on $\R^2$ ******************* 




#************************ Monte-Carlo experiment: **************************
##B-->simulated sample & nk-->estimates for k=kl:ku

 xp         = matrix(0,B,nk);
 xp_als     = matrix(0,B,nk);
 qmes       = matrix(0,B,nk);

#Monte-Carlo trials:
for (b in 1:B)
{

#                    #-------- (1)  simulated data:  ---------#
#                    #    when (X,Y) has a joint Student t(df)  
#                    #    distribution on $(0,\infty)^2$ 
#                    #----------------------------------------#
#                    #require(mvtnorm)
#                    Z1_Z2_tab = rmvt(n, sigma = diag(2), df=df , delta = rep(0, 2), type = "shifted");
#                    Z1tab = Z1_Z2_tab[,1];     
#                    Z2tab = Z1_Z2_tab[,2];  
#                    xtab = abs(Z1tab);      
#                    ytab  = abs(Z2tab);
#                    #-------------------------------------#    
	
	
		
#                    #--------  (2) simulated data:  ---------#
#                    #    when (X,Y) has a positive cauchy   
#                    #    distribution on $(0,\infty)^2$ 
#                    #----------------------------------------#
#                    #require(mvtnorm)
#                    #(Z1,Z2) denotes a standard Cauchy distribution on R2:                    
#                    Z1_Z2_tab = rmvt(n, sigma = diag(2), df=1, delta = rep(0, 2), type = "shifted");
#                    Z1tab = Z1_Z2_tab[,1];     
#                    Z2tab = Z1_Z2_tab[,2]; 
#                     
#                    xtab = (abs(Z1tab))^(2/5);      
#                    ytab  = abs(Z2tab);                    
#                   #-------------------------------------#    


  
 
#                    #--------  (3) simulated data:  ---------#
#                    #    when (X,Y) has a general cauchy   
#                    #    distribution on $\R^2$ 
#                    #----------------------------------------#
#                    #require(mvtnorm)
#                    #(Z1,Z2) denotes a standard Cauchy distribution on R2:
#                    Z1_Z2_tab = rmvt(n, sigma = diag(2), df=1, delta = rep(0, 2), type = "shifted");
#                    Z1tab = Z1_Z2_tab[,1];     
#                    Z2tab = Z1_Z2_tab[,2]; 
#                     
#                    #xtab = (Z1tab^(2/5)*(Z1tab>=0))+(Z1tab^(1/5)*(Z1tab<0));      
#                    #ytab = (Z2tab*(Z1tab>=0)) + (Z2tab^(1/3)*(Z1tab<0));   
#
#                    xtab = (sign(Z1tab)*(abs(Z1tab))^(2/5)*(Z1tab>=0))+(sign(Z1tab)*(abs(Z1tab))^(1/5)*(Z1tab<0));      
#                    ytab = (Z2tab*(Z1tab>=0)) + (sign(Z2tab)*(abs(Z2tab))^(1/3)*(Z1tab<0));                                       
#                   #-------------------------------------#    

 
 
                    #--------  (4) simulated data:  ---------#
                    #    when (X,Y) has a Transformed Student   
                    #    t(df) distribution on $\R^2$ 
                    #----------------------------------------#

                    #require(mvtnorm)
                    #(Z1,Z2) denotes a standard Student t(df) distribution on R2:
                    Z1_Z2_tab = rmvt(n, sigma = diag(2), df=df, delta = rep(0, 2), type = "shifted");
                    Z1tab = Z1_Z2_tab[,1];     
                    Z2tab = Z1_Z2_tab[,2]; 
                                         
                    # X = (Z1^(df/4)*(Z1>=0)) - ( (-Z1)^(df/4)*(Z1<0)) :
                    xtab = (sign(Z1tab)*(abs(Z1tab))^(df/4)*(Z1tab>=0))+(sign(Z1tab)*(abs(Z1tab))^(df/4)*(Z1tab<0));   
                    # Y = Z2 :        
                    ytab = Z2tab;  
                    
                    #-------------------------------------#    

                                             
 for (i in 1:nk)
     {      
              	 #sample fraction:  
             	 k=kk[i];
            	 
         	     #Hill's EVI:
        	     gamY = Hill_inverse(ytab,k);
                 #gamY = min(gamY,1/2);

                 #QMES'estimate with \alpha_n=1-pn:
                 qmes[b,i]  = QMESh(xtab,ytab,pn,k)


                 #Estimate $\widehat\tau_n'(\alpha_n)$ of the extreme expectile level $\tau_n'(\alpha_n)$:
                 htau = 1 - ((pn) * (gamY/(1-gamY)));

               	 
               	 #Indirect expectile-MES $\widehat{\textrm{XMES}}^*(\tau_n'(\alpha_n))$: 
                 xp[b,i] = XMESh(xtab,ytab,(1-htau),k);
                 
                                     
                 ##Direct expectile-MES $\widetilde{\textrm{XMES}}^*(\tau_n'(\alpha_n))$: 
                 xp_als[b,i] = XMESt(xtab,ytab,(1-htau),k);           
                 
      }
} 
         








#____________________________________________________
#
#  (9.5)  MSE & Bias estimates: Figures 5-6 in the main paper
#                      
#____________________________________________________


xp_als_MSE = colMeans((tXMES - xp_als)^2);    #Direct (LAWS) Xpectile estimator
                                  
xp_MSE = colMeans((tXMES - xp)^2,na.rm=TRUE); #Indirect (plug-in) Xpectile estimator



#--------------------------------------
# Normalized Root-MSE & Bias estimates:                   
#--------------------------------------

xp_als_NMSE = sqrt(xp_als_MSE)/abs(tXMES[1,1]);
xp_NMSE     = sqrt(xp_MSE)/abs(tXMES[1,1]);  
        # kk = kk[2:154];
        # qu_ppn_MSE = sqrt(qu_ppn_MSE[2:154]);
        # xp_als_MSE = sqrt(xp_als_MSE[2:154]);
        # xp_MSE = sqrt(xp_MSE[2:154]);




xp_Bias      = colMeans(xp - tXMES,na.rm=TRUE);      
xp_als_Bias  = colMeans(xp_als - tXMES);
       # xp_als_Bias = xp_als_Bias[2:154];
       # xp_Bias = xp_Bias[2:154];
xp_NBias     = xp_Bias/tXMES[1,1];
xp_als_NBias = xp_als_Bias/tXMES[1,1];
       
       
       
#-----------------------------------
#           Save/Load data:
#-----------------------------------

###### Positive Student:
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//t3_n1000.RData")
#
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//t3_n1000.RData")
#######
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//t5_n1000.RData")
#
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//t5_n1000.RData")
#######
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//t7_n1000.RData")
#
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//t7_n1000.RData")
#######
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//t9_n1000.RData")
#
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//t9_n1000.RData")
#######



###### Transformed Student:
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//transformed_t3_n1000.RData")
#
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//transformed_t3_n1000.RData")
#
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//transformed_t5_n1000.RData")
#
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//transformed_t5_n1000.RData")
#
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//transformed_t7_n1000.RData")
#
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//transformed_t7_n1000.RData")
#
#save(kk,  xp_als_Bias,  xp_Bias, xp_als_NBias, xp_NBias,  xp_als_MSE, xp_MSE, xp_als_NMSE, xp_NMSE,file="/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//transformed_t9_n1000.RData")
#
#load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//transformed_t9_n1000.RData")



#-----------------------------------
#             Graphs:
#-----------------------------------

df2_NMSE <- data.frame(kk,xp_als_NMSE,xp_NMSE)             
names(df2_NMSE) <- c("Sample_fraction_k","LAWS","Plug-in")


df2_NBIAS <- data.frame(kk,xp_als_NBias,xp_NBias)      
names(df2_NBIAS) <- c("Sample_fraction_k","LAWS","Plug-in")
     
       
dev.new()
ecm <- melt(df2_NMSE, id = "Sample_fraction_k")
dd <- qplot(Sample_fraction_k, value, data=ecm, geom="line", colour=variable)+geom_line(size = 4, linetype=1)
dd+scale_color_manual(values = c("LAWS" = '#3399ff',"Plug-in" = '#FA8072')) +
scale_y_continuous("root MSE") +
 scale_x_continuous("Sample fraction k")+
labs(title = "Student t(9)") #--------------Modify "df" here---------------


dev.new()
ecm <- melt(df2_NBIAS, id = "Sample_fraction_k")
dd <- qplot(Sample_fraction_k, value, data=ecm, geom="line", colour=variable)+geom_line(size = 4, linetype=2)
dd+scale_color_manual(values = c("LAWS" = 'blue',"Plug-in" = 'red')) +
scale_y_continuous("BIAS") +
 scale_x_continuous("Sample fraction k")+
labs(title = "Student t(9)") #--------------Modify "df" here---------------




#----- Black and white (MSE):
dev.new()
df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NMSE)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NMSE) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = 1.5, linetype=1) 
dd+geom_line(size = 1.5, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("RMSE") +
scale_x_continuous("k")



#----- Black and white (BIAS):
dev.new()
df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NBias)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NBias) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = 1, linetype=1) 
dd+geom_line(size = 2, data = df_xp, colour = "black", linetype=3) +
scale_y_continuous("BIAS") +
scale_x_continuous("k")




#---------------------------------------------------------------------------------------------------
#
#
#    Figure 5 in the main paper:  fig5_dgs_jrssb.pdf
#
#                             
#---------------------------------------------------------------------------------------------------
dev.new()

require(grid)
grid.newpage()
pushViewport(viewport(layout = grid.layout(2,2)))

define_region = function(row, col){
	viewport(layout.pos.row=row, layout.pos.col=col)
}

#-------------
# RMSE df=3
#-------------
load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//t3_n1000.RData")

df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NMSE)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NMSE) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = 1, linetype=1) 
print(dd+geom_line(size = 1, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("RMSE") +
scale_x_continuous("k")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(1,1))


#-------------
# RMSE df=5
#-------------
load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//t5_n1000.RData")

df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NMSE)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NMSE) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = 1, linetype=1) 
print(dd+geom_line(size = 1, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("RMSE") +
scale_x_continuous("k")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(1,2))


#-------------
# Bias df=3
#-------------
load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//t3_n1000.RData")

df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NBias)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NBias) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = 1, linetype=1) 
print(dd+geom_line(size = 1, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("BIAS") +
scale_x_continuous("k")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(2,1))


#-------------
# Bias df=5
#-------------
load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//t5_n1000.RData")

df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NBias)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NBias) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = 1, linetype=1) 
print(dd+geom_line(size = 1, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("BIAS") +
scale_x_continuous("k")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(2,2))




#---------------------------------------------------------------------------------------------------
#
#
#    Figure 6 in the main paper:  fig6_dgs_jrssb.pdf
#
#                             
#---------------------------------------------------------------------------------------------------
dev.new()

require(grid)
grid.newpage()
pushViewport(viewport(layout = grid.layout(2,2)))

define_region = function(row, col){
	viewport(layout.pos.row=row, layout.pos.col=col)
}

#-------------
# RMSE df=3
#-------------
load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//transformed_t3_n1000.RData")

df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NMSE)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NMSE) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = 1, linetype=1) 
print(dd+geom_line(size = 1, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("RMSE") +
scale_x_continuous("k")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(1,1))


#-------------
# RMSE df=5
#-------------
load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//transformed_t5_n1000.RData")

df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NMSE)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NMSE) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = 1, linetype=1) 
print(dd+geom_line(size = 1, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("RMSE") +
scale_x_continuous("k")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(1,2))


#-------------
# Bias df=3
#-------------
load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//transformed_t3_n1000.RData")

df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NBias)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NBias) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("BIAS") +
scale_x_continuous("k")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(2,1))


#-------------
# Bias df=5
#-------------
load("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles//transformed_t5_n1000.RData")

df_xp_als  <- data.frame(xxc = kk, yyc = xp_als_NBias)              
    df_xp  <- data.frame(xxc = kk, yyc = xp_NBias) 

dd <- ggplot(data=df_xp_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xp, colour = "black", linetype=2) +
scale_y_continuous("BIAS") +
scale_x_continuous("k")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(2,2))










#____________________________________________________
#
#   (9.6)  QQ-plots in Supplement (A.2) :
#
#____________________________________________________
   
#sample size:
n = 1000;

#order "pn < 1/n" [expectile_order = 1-pn]:
    if (n==100){pn = 0.005;}      
    if (n==1000){pn = 0.0006;}    
                                         
 
#grid of values of "k":
kl = floor(log(n^0.9))+1;  
     if (n==100){ku = 17;}     
     if (n==1000){ku = floor(n/log(n^0.9));} 
kk = (kl+2):ku; 
nk = length(kk);
 
#Nb of simulated samples:
     if (n==100){B = 10000;}     
     if (n==1000){B = 10000;} #2000;}



#************************   True XMES: ************************************** 

#------------------------------------------      
                  df = 9;  
#------------------------------------------ 

  tXVaR = expectile_transformed_student(df, 1-pn, linf=0, lsup=50); 
  tXVaR = expectile_transformed_student(df, 1-pn, linf=(tXVaR$minimum - 2), lsup=(tXVaR$minimum + 2));
  
  tXMES = True_MES_student(df,tXVaR$minimum);
  tXMES = tXMES * matrix(1,B,nk); 

#************************  End True XMES ************************************** 


#************************ Monte-Carlo experiment: **************************
##B-->simulated sample & nk-->estimates for k=kl:ku

 xp         = matrix(0,B,nk);
 xp_als     = matrix(0,B,nk);

xp_laws_opt = 0 * c(1:B);
xp_plug_opt = 0 * c(1:B);

Wntilde     = matrix(0,B,nk);;
Wnhat       = matrix(0,B,nk);;

kopt_laws   =  0 * c(1:B);
kopt_plug   =  0 * c(1:B);


A_fct <- function(s,df)
    {
       n = df;
       num = 2*gamma((n+1)/2) * n^((n-1)/2);
       den = sqrt(n*pi) * gamma(n/2);      
       #c_n=\frac{2 \Gamma((n+1)/2) n^{(n-1)/2}}{\sqrt{n\pi} \Gamma(n/2)} :
       c_n = num/den;
       #A(s)\sim \frac{n+1}{n+2} (c_n s)^{-2/n} :
       res = ((n+1)/(n+2)) * (c_n * s)^(-2/n);
       return(res);
     }
 

#Monte-Carlo trials:
for (b in 1:B)
{

                   #--------   simulated data:  ---------#
                   #    when (X,Y) has a joint Student   
                   #    distribution on $(0,\infty)^2$ 
                   #-------------------------------------#
                   #require(mvtnorm)
                   Z1_Z2_tab = rmvt(n, sigma = diag(2), df = df, delta = rep(0, 2), type = "shifted");
                   Z1tab = Z1_Z2_tab[,1];     
                   Z2tab = Z1_Z2_tab[,2];  
                   xtab = abs(Z1tab);      
                   ytab  = abs(Z2tab);
                   
                   # #Joint density (for making some tests):   
                   # dmvt(c(0,0), delta = rep(0, 2), sigma = diag(2), df = 9, log = FALSE,  type = "shifted")
                   #-------------------------------------#    
	
	
	
	
                   # #--------   simulated data:  ---------#
                   # #    when (X,Y) has a joint Student   
                   # #    distribution on $\R^2$ 
                   # #-------------------------------------#
                   
                   # tbm ??
                   
                   # #require(mvtnorm) 
                   # Z1_Z2_tab = rmvt(n, sigma = diag(2), df = df, delta = rep(0, 2), type = "shifted");
                   # Z1tab = Z1_Z2_tab[,1];     
                   # Z2tab = Z1_Z2_tab[,2];  
                   # xtab = abs(Z1tab);      
                   # ytab  = abs(Z2tab);
                   
                   # # #Joint density (for making some tests):   
                   # # dmvt(c(0,0), delta = rep(0, 2), sigma = diag(2), df = 9, log = FALSE,  type = "shifted")
                   # #-------------------------------------#    

 
 
                                              
 for (i in 1:nk)
     {      
              	 #sample fraction:  
            	 k=kk[i];
               	 
               	 #Indirect expectile-MES $\widehat{\textrm{XMES}}^*(\tau'_n)$: 
                 xp[b,i] = XMESh(xtab,ytab,pn,k);
                                     
                 ##Direct expectile-MES $\widetilde{\textrm{XMES}}^*(\tau'_n)$: 
                 xp_als[b,i] = XMESt(xtab,ytab,pn,k);               
                
         truexp      = tXMES[1,1];  
         gama        = 1/df;
         rrho        = -2/df;  
                             
         lambda2    =  sqrt(k) * A_fct((n/k),df);      
         rn         =  sqrt(k)/(log(k/(n*pn)));   
                
         #Wntilde[b]   = ( (rn * ((xp_als[b,i]/truexp)-1))  - (lambda2/(1-rrho)) )/gama;     
         Wntilde[b,i]  = ( (rn * log(xp_als[b,i]/truexp))  - (lambda2/(1-rrho)) )/gama;   
                 
         #Wntilde[b] = ( (rn * ((xp[b,i]/truexp)-1))  - (lambda2/(1-rrho)) )/gama;   
         Wnhat[b,i]  = ( (rn * log(xp[b,i]/truexp))  - (lambda2/(1-rrho)) )/gama;                                                 
      }
}


#---------------------------------------------------------------------------------------------------
####                         The optimal "k" minimizing the MSE:                                ####
#---------------------------------------------------------------------------------------------------

xp_als_MSE = colMeans((tXMES - xp_als)^2);    #Direct (LAWS) Xpectile estimator
         #
         ii_xp_als   = which (xp_als_MSE==min(xp_als_MSE));
         k_tilde        = kk[ii_xp_als];
         
                 
         
xp_MSE = colMeans((tXMES - xp)^2,na.rm=TRUE); #Indirect (plug-in) Xpectile estimator
         #
         ii_xp       = which (xp_MSE==min(xp_MSE));
         k_hat         = kk[ii_xp];         
         
                  
k_hat

k_tilde   


#____________________________________________________
#
#  (9.7)  QQ-plots: Figures 16-17 in Supplement (A.2)
#                      
#____________________________________________________


# qqnorm(Wntilde[,ii_xp_als], main="LAWS", xlab="Theoretical Quantiles", ylab="Sample Quantiles", col=4)
# qqline(Wntilde[,ii_xp_als], col = 1, lwd = 2)

# qqnorm(Wnhat[,ii_xp], main="Plug-in", xlab="Theoretical Quantiles", ylab="Sample Quantiles", col=2)
# qqline(Wnhat[,ii_xp], col = 1, lwd = 2)


qplot(sample = Wnhat[,ii_xp], stat="qq")+
labs(title="t(9) , Plug-in , k=13")


qplot(sample = Wntilde[,ii_xp_als], stat="qq")+
labs(title="t(9) , LAWS , k= 9")

#---------------------------------------------------------------------------------------------------











#-------------------------------------------------------------------------------------------
#
#       (10)    Section 8 :  Applications
#
#-------------------------------------------------------------------------------------------


#__________________________________
#
#     (10.1)   Load SOA data:
#
#__________________________________

#first step:
don<-read.table("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles/R_ggplot/soa.m", sep=",",dec=".", skip=1)
#second step:
don<-read.table("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles/R_ggplot/soa.m", sep=",",dec=".", skip=1,nrows=75789)

Claims <- don$V1
ytab <- don$V1



#__________________________________
#
#   (10.2)   Histogram of the log claims (Figure 7 top):
#
#__________________________________

m <- ggplot(don, aes(x=log(Claims)))
m + geom_histogram(aes(fill = ..count..)) +
 scale_fill_gradient("Count", low = "red", high = "yellow")+ geom_rug(sides="b",colour = "blue")

# Black & white version :

m <- ggplot(don, aes(x=log(Claims)))
m + geom_histogram(colour = "black", fill = "white") +
geom_rug(sides="b",colour = "black")+
labs(title = "(a) Histogram")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black"))







#__________________________________
#
#    (10.3)   Calculation of the VaR estimates:
#
#__________________________________

##Sample size:
 n = length(ytab);
 Yn = max(ytab);
 ysort=sort(ytab);   

##order "pn < 1/n=0.0286":
 pn = 1/100000;

##grid of values of "k":
 kl = 103; ku = 1000
 kk = kl:ku; 
 nk = length(kk);



##Initialization of VaR & ES vectors:
gam =  cbind(rep(0, nk));      #Hill's EVI
rho =  cbind(rep(0, nk));      #Inverse of Hill's EVI

qu  = cbind(rep(0, nk));       #quantile-VaR
qes = cbind(rep(0, nk));       #quantile-ES

xp = cbind(rep(0, nk));        #Indirect expectile-VaR
xp_als = cbind(rep(0, nk));    #Direct expectile-VaR

hxes_star = cbind(rep(0, nk));   #1st-Indirect expectile-ES
hxes_dagger = cbind(rep(0, nk)); #2nd-Indirect expectile-ES
txes_star = cbind(rep(0, nk));   #1st-Direct expectile-ES
txes_dagger = cbind(rep(0, nk)); #2nd-Direct expectile-ES

line_max = cbind(rep(0, nk));    #Sample maximum



##Calculation of VaR & ES vectors:
for (i in 1:nk)
        { 
        	k=kk[i];
        	#Sample maximum:
        	line_max[i] = Yn; 
        	#Hill's EVI:
        	gam[i] = Hill_inverse(ytab,kk[i]);
        	     #Inverse of Hill's EVI:
        	     rho[i]= 1/gam[i];
              
              #quantile-VaR $\hat{q}_{1-p_n}^{\star}$: 
              qu[i]=Weissman_quantile(ytab,pn,k);
              
              #quantile-ES $\widehat{\textrm{QES}}^{\star}(1-p_n)$:   
              qes[i] = mean(ytab*(ytab>ysort[n-k]))/(k/n) * (k/(n*pn))^(gam[i]);
              
              
              #Estimate $\widehat\tau_n'(\alpha_n)$ of the extreme expectile level $\tau_n'(\alpha_n)$:
              htau = 1 - ((pn) * (gam[i]/(1-gam[i])));
              
              
              #Indirect expectile-VaR $\hat\xi_{1-p_n}^{\star}$:
              xp[i] = (rho[i] - 1)^(-gam[i]) * Weissman_quantile(ytab,(1-htau),k);
             
                    
              #Direct expectile-VaR $\widetilde\xi_{1-p_n}^{\star}$:
              xp_als[i] = (n*(1-htau)/k)^(-gam[i]) * expectile(ytab,probs=1-(k/n)); #here we use the "expectreg" package
              
               
              #1st-Indirect expectile-ES $\widehat{\textrm{XES}}^{\star}(1-p_n)$ :
              hxes_star[i] = (1-gam[i])^(-1) * xp[i];
              
              
              #1st-Direct expectile-ES $\widetilde{\textrm{XES}}^{\star}(1-p_n)$ :
              txes_star[i] = (1-gam[i])^(-1) * xp_als[i];
              
                          
              #2nd-Indirect expectile-ES $\widehat{\textrm{XES}}^{\dagger}(1-p_n)$:              
              qesi = mean(ytab*(ytab>ysort[n-k]))/(k/n) * (k/(n*(1-htau)))^(gam[i]);              
              hxes_dagger[i] = (xp[i]/Weissman_quantile(ytab,(1-htau),k)) * qesi;
              
                            
              #2nd-Direct expectile-ES $\widetilde{\textrm{XES}}^{\dagger}(1-p_n)$ :
              txes_dagger[i] = (xp_als[i]/Weissman_quantile(ytab,(1-htau),k)) * qesi;
              
           }
         
         



#_______________________________________________________________
#
#    (10.4)   Graphs of the estimated VaR (Figure 7):
#
#_______________________________________________________________

#---------------------------------------------------------------------  
#   Hist + VaR in a same figure :
#       fig7_dgs_jrssb.pdf  
#---------------------------------------------------------------------    
   
dev.new()
            
df <-data.frame(xxc=kk, yyc = qu, gam)              # rainbow                     
dfff <-data.frame(xxc = kk, yyc = line_max)         # pink                    
dff <-data.frame(xxc = kk, yyc = qes, gam)          # red                                   
df5 <-data.frame(xxc = kk, yyc = hxes_star, gam)    # gray              
df7 <-data.frame(xxc = kk, yyc = xp_als, gam)       # yellow
df8 <-data.frame(xxc = kk, yyc = txes_star, gam)    # cyan
df9 <-data.frame(xxc = kk, yyc = txes_dagger, gam)  # black                    
      
#require(grid)
grid.newpage()
pushViewport(viewport(layout = grid.layout(2,3)))

define_region = function(row, col){
	viewport(layout.pos.row=row, layout.pos.col=col)
}

m <- ggplot(don, aes(x=log(Claims)))

print(m + geom_histogram(colour = "black", fill = "white") +
geom_rug(sides="b",colour = "black")+
labs(title = "(a) Histogram")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
vp=define_region(1,1:3))


dd <- ggplot(data=df, aes(xxc, yyc)) + geom_line(size = 1, colour = "grey") 

print(dd+geom_line(size = 1,data = dfff, colour = "black", linetype=1)+
#geom_line(size = 2,data = dfff, colour = "black", linetype=3)+
scale_y_continuous(name = "Risk  Measure")+ 
scale_x_continuous("Sample  fraction  k", limit = c(103, 800))+
geom_line(size = 1,data = df7, colour = "black", linetype=2)+
labs(title = "(b) VaR plots")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")) ,       
  vp=define_region(2,1:3))




############################  Estimates on the stable region $k\in[150,500]$ ################    


ind150 = which(kk==150)  # 48
ind500 = which(kk==500)  # 398

# rainbow curve :
c( min(qu[ind150:ind500])  ,  max(qu[ind150:ind500]) )

# yellow curve :
c( min(xp_als[ind150:ind500])  ,  max(xp_als[ind150:ind500]) )

# red curve : 
mean( qes[ind150:ind500] )

# gray curve : 
mean( hxes_star[ind150:ind500] )

# cyan curve : 
mean( txes_star[ind150:ind500] )
   
# black curve : 
mean( txes_dagger[ind150:ind500] )
   







#_______________________________________________________________
#
#    (10.5)   Load "daily" and "weekly" data:
#
#_______________________________________________________________
     
          
# sample size:     
n=2513;

bd=read.csv("/Users/daouia/Desktop/Daouia_Gilles_Stephane/R_matlab_xpectiles/Broker_Dealers_new.csv",header=T);

Y=bd[1:n,c(1,8)];#date and market index return
GS=bd[which(bd[,4]==86868),c(1,7)]; # date and GS stock return
MS=bd[which(bd[,4]==69032),c(1,7)];# date and MS stock return
TROW=bd[which(bd[,4]==10138),c(1,7)];#date and TROW stock return


# minus daily log return (daily loss):
Y[,2]=-log(1+Y[,2]);
lgs=-log(1+GS[,2]);
lms=-log(1+MS[,2]);
ltrow=-log(1+TROW[,2]);

YX=cbind(Y[,2],lgs,lms,ltrow);
YX=YX[order(YX[,1],decreasing=TRUE),];



#-------- Choose  "daily" data --------:   

#Market index    
 ytab = Y[,2];
   
#Goldman Sachs (comlumn "Y"):
 xtab1 = lgs;
 
#Morgan Stanley (comlumn "P"):
 xtab2 = lms;
 
#T. Rowe Price (comlumn "G"):
 xtab3 = ltrow;
 
#--------------------- End daily 
 
 
  
  
#-------- Or choose "weekly" data --------:  

dats=bd[1:n,1]
dats=data.frame(date=as.character(dats))

dats= as.Date(strptime(dats$date, "%Y%m%d"))
weekday=weekdays(dats);
weeknum=floor((as.numeric(dats)-as.numeric(dats[1]))/7)+1;

#compute the weekly loss
j=1
ttlweeknum=weeknum[n];
Yw=vector();
GSw=vector();
MSw=vector();
TROWw=vector();
weekno=vector();
for(i in 1:ttlweeknum)
{
  ind=which(weeknum==i);
  weekno[j]=weeknum[i];
  Yw[j]=sum(Y[ind,2]);
  GSw[j]=sum(lgs[ind]);
  MSw[j]=sum(lms[ind]);
  TROWw[j]=sum(ltrow[ind]);
  weekno[j]=i;
  j=j+1
  
}

YXw=cbind(Yw,GSw,MSw,TROWw,weekno);
YXw=YXw[order(YXw[,1],decreasing=TRUE),]; 
  
ytab=Yw; 

xtab1=GSw; 

xtab2=MSw; 

xtab3=TROWw; 

#--------------------------------- End weekly  
  
  







#__________________________________
#
# (10.6) Calculations of the estimates of $\gamma_X$, $\gamma_Y$ and QMES and XMES (Supplement):
#     
#__________________________________

##Sample size:
n = length(ytab);   

##order "pn = 1/n":
pn = 1/n;

##grid of values of "k":
kl = 1; 
ku = 300;
kk = kl:ku; 
nk = length(kk);

##Hill's estimates:
gamY  = cbind(rep(0, nk));  
gamX1 = cbind(rep(0, nk)); 
gamX2 = cbind(rep(0, nk));
gamX3 = cbind(rep(0, nk));

##MES's estimates for X1=xtab1:
QMES_H1  = cbind(rep(0, nk));  
XMES_H1  = cbind(rep(0, nk)); 
XMES_T1  = cbind(rep(0, nk));

##MES's estimates for X2=xtab2:
QMES_H2  = cbind(rep(0, nk));  
XMES_H2  = cbind(rep(0, nk)); 
XMES_T2  = cbind(rep(0, nk));

##MES's estimates for X3=xtab3:
QMES_H3  = cbind(rep(0, nk));  
XMES_H3  = cbind(rep(0, nk)); 
XMES_T3  = cbind(rep(0, nk));


for (i in 1:nk)
        { 
        	#sample fraction: 
        	k=kk[i];
        	
        	#Hill's EVI:
        	gamY[i]  = Hill_inverse(ytab,k);
        	gamX1[i] = Hill_inverse(xtab1,k);
        	gamX2[i] = Hill_inverse(xtab2,k);
        	gamX3[i] = Hill_inverse(xtab3,k);

            #QMES'estimates:
            QMES_H1[i]  = QMESh(xtab1,ytab,pn,k)
            QMES_H2[i]  = QMESh(xtab2,ytab,pn,k)
            QMES_H3[i]  = QMESh(xtab3,ytab,pn,k)

            #XMES_H'estimates:
            XMES_H1[i]  = XMESh(xtab1,ytab,pn,k)
            XMES_H2[i]  = XMESh(xtab2,ytab,pn,k)
            XMES_H3[i]  = XMESh(xtab3,ytab,pn,k)

            #XMES_T'estimates:
            XMES_T1[i]  = XMESt(xtab1,ytab,pn,k)
            XMES_T2[i]  = XMESt(xtab2,ytab,pn,k)
            XMES_T3[i]  = XMESt(xtab3,ytab,pn,k)
        }
         







#__________________________________
#
# (10.7) Plot of the MES estimates:
#     Fig 15a (Supplement): Goldman_Sachs_daily_s
#     
#__________________________________

k150= which(kk==150)

dev.new()

df <-data.frame(xxc=kk[1:k150], yyc = XMES_H1[1:k150], gamX1[1:k150])  
dff <-data.frame(xxc=kk[1:k150], yyc = XMES_T1[1:k150], gamX1[1:k150]) 
dfff <-data.frame(xxc=kk[1:k150], yyc = QMES_H1[1:k150], gamX1[1:k150]) 
#
dd <- ggplot(data=dfff, aes(xxc, yyc,col=gamX1[1:k150])) + geom_line(size = 2) 
dd+geom_line(size = 2.25, data = dff, colour = "black", linetype=1)+
geom_line(size = 1.75, data = df, colour = "HotPink1", linetype=1)+
geom_line(size = 1, data = df, colour = "pink", linetype=1)+
#
#scale_colour_gradientn(name="gamma_X", colours=rainbow(7)) + 
scale_colour_gradientn(name=expression(hat(gamma)[X]), colours=rainbow(7)) + 
scale_y_continuous(name = "MES")+  #, limit = c(3*10^6,7.3*10^6))+
scale_x_continuous("Sample  fraction  k")+  #, limit = c(85, 105))+
labs(title = "(a) Goldman Sachs")



            ######  black & white version :

dev.new()

df <-data.frame(xxc=kk[1:k150], yyc = XMES_H1[1:k150])  
dff <-data.frame(xxc=kk[1:k150], yyc = XMES_T1[1:k150]) 
dfff <-data.frame(xxc=kk[1:k150], yyc = QMES_H1[1:k150]) 
#
dd <- ggplot(data=dfff, aes(xxc, yyc)) + geom_line(size = 1, linetype=3) 
dd+geom_line(size = 1.5, data = dff, colour = "grey", linetype=1)+
geom_line(size = 1, data = df, colour = "black", linetype=2)+
#
scale_y_continuous(name = "MES")+  #, limit = c(3*10^6,7.3*10^6))+
scale_x_continuous("Sample  fraction  k")+  #, limit = c(85, 105))+
labs(title = "(a) Goldman Sachs")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black"))










#__________________________________
#
# (10.8) Averaged MES estimates for Goldman Sachs:
#      
#__________________________________

#The first stable region appears for k\in [k_l, k_u]: 
k_l=85; 
k_u=105; 
ind_l = which(kk==k_l);
ind_u = which(kk==k_u);
#Averaged estimate:
c( mean(XMES_H1[ind_l:ind_u])  , mean(XMES_T1[ind_l:ind_u])  ,  mean(QMES_H1[ind_l:ind_u]) )





#__________________________________
#
# (10.9) Plot of the MES estimates:
#      Fig 15b (Supplement): Morgan_Stanley_daily_s 
#     
#__________________________________

dev.new()

df <-data.frame(xxc=kk[1:k150], yyc = XMES_H2[1:k150], gamX2[1:k150])  
dff <-data.frame(xxc=kk[1:k150], yyc = XMES_T2[1:k150], gamX2[1:k150]) 
dfff <-data.frame(xxc=kk[1:k150], yyc = QMES_H2[1:k150], gamX2[1:k150]) 
#
dd <- ggplot(data=dfff, aes(xxc, yyc,col=gamX2[1:k150])) + geom_line(size = 2) 
dd+geom_line(size = 2.25, data = dff, colour = "black", linetype=1)+
geom_line(size = 1.75, data = df, colour = "HotPink1", linetype=1)+
geom_line(size = 1, data = df, colour = "pink", linetype=1)+
#
#scale_colour_gradientn(name="gamma_X", colours=rainbow(7)) + 
scale_colour_gradientn(name=expression(hat(gamma)[X]), colours=rainbow(7)) + 
scale_y_continuous(name = "MES")+  #, limit = c(3*10^6,7.3*10^6))+
scale_x_continuous("Sample  fraction  k")+  #, limit = c(85, 115))+
labs(title = "(b) Morgan Stanley")



            ######  black & white version :

dev.new()

df <-data.frame(xxc=kk[1:k150], yyc = XMES_H2[1:k150], gamX2[1:k150])  
dff <-data.frame(xxc=kk[1:k150], yyc = XMES_T2[1:k150], gamX2[1:k150]) 
dfff <-data.frame(xxc=kk[1:k150], yyc = QMES_H2[1:k150], gamX2[1:k150]) 
#
dd <- ggplot(data=dfff, aes(xxc, yyc)) + geom_line(size = 1, linetype=3) 
dd+geom_line(size = 1.5, data = dff, colour = "grey", linetype=1)+
geom_line(size = 1, data = df, colour = "black", linetype=2)+
#
scale_y_continuous(name = "MES")+  #, limit = c(3*10^6,7.3*10^6))+
scale_x_continuous("Sample  fraction  k")+  #, limit = c(85, 105))+
labs(title = "(b) Morgan Stanley")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black"))







#__________________________________
#
# (10.10) Averaged MES estimates for Morgan Stanley:
#      
#__________________________________

#The first stable region appears for k\in [k_l, k_u]: 
k_l=85; 
k_u=115; 
ind_l = which(kk==k_l);
ind_u = which(kk==k_u);
#Averaged estimate:
c( mean(XMES_H2[ind_l:ind_u])  , mean(XMES_T2[ind_l:ind_u])  ,  mean(QMES_H2[ind_l:ind_u]) )






#__________________________________
#
# (10.11) Plot of the MES estimates:
#      Fig 15c (Supplement): TRowe_Price_daily_s 
#     
#__________________________________

dev.new()

df <-data.frame(xxc=kk[1:k150], yyc = XMES_H3[1:k150], gamX3[1:k150])  
dff <-data.frame(xxc=kk[1:k150], yyc = XMES_T3[1:k150], gamX3[1:k150]) 
dfff <-data.frame(xxc=kk[1:k150], yyc = QMES_H3[1:k150], gamX3[1:k150]) 
#
dd <- ggplot(data=dfff, aes(xxc, yyc,col=gamX3[1:k150])) + geom_line(size = 2) 
dd+geom_line(size = 2.25, data = dff, colour = "black", linetype=1)+
geom_line(size = 1.75, data = df, colour = "HotPink1", linetype=1)+
geom_line(size = 1, data = df, colour = "pink", linetype=1)+
#
#scale_colour_gradientn(name="gamma_X", colours=rainbow(7)) + 
scale_colour_gradientn(name=expression(hat(gamma)[X]), colours=rainbow(7)) + 
scale_y_continuous(name = "MES")+  #, limit = c(3*10^6,7.3*10^6))+
scale_x_continuous("Sample  fraction  k")+  #, limit = c(70, 100))+
labs(title = "(c) T. Rowe Price")

            ######  black & white version :

dev.new()

df <-data.frame(xxc=kk[1:k150], yyc = XMES_H3[1:k150], gamX3[1:k150])  
dff <-data.frame(xxc=kk[1:k150], yyc = XMES_T3[1:k150], gamX3[1:k150]) 
dfff <-data.frame(xxc=kk[1:k150], yyc = QMES_H3[1:k150], gamX3[1:k150]) 
#
dd <- ggplot(data=dfff, aes(xxc, yyc)) + geom_line(size = 1, linetype=3) 
dd+geom_line(size = 1.5, data = dff, colour = "grey", linetype=1)+
geom_line(size = 1, data = df, colour = "black", linetype=2)+
#
scale_y_continuous(name = "MES")+  #, limit = c(3*10^6,7.3*10^6))+
scale_x_continuous("Sample  fraction  k")+  #, limit = c(85, 105))+
labs(title = "(c) T. Rowe Price")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black"))





#__________________________________
#
# (10.12) Averaged MES estimates for T. Rowe Price:
#      
#__________________________________

#The first stable region appears for k\in [k_l, k_u]: 
k_l=70;
k_u=100;
ind_l = which(kk==k_l);
ind_u = which(kk==k_u);
#Averaged estimate:
c( mean(XMES_H3[ind_l:ind_u])  , mean(XMES_T3[ind_l:ind_u])  ,  mean(QMES_H3[ind_l:ind_u]) )





#######################  The 3 banks in a same figure : ###################################    
#    Figure  " three_banks_fig15_bn "
###########################################################################################
   
dev.new()

#require(grid)
grid.newpage()
pushViewport(viewport(layout = grid.layout(3,3)))

define_region = function(row, col){
	viewport(layout.pos.row=row, layout.pos.col=col)
}

k150= which(kk==150)

df <-data.frame(xxc=kk[1:k150], yyc = XMES_H1[1:k150])  
dff <-data.frame(xxc=kk[1:k150], yyc = XMES_T1[1:k150]) 
dfff <-data.frame(xxc=kk[1:k150], yyc = QMES_H1[1:k150]) 
dd <- ggplot(data=dfff, aes(xxc, yyc)) + geom_line(size = 1, linetype=3) 
print(dd+geom_line(size = 1.5, data = dff, colour = "grey", linetype=1)+
geom_line(size = 1, data = df, colour = "black", linetype=2)+
scale_y_continuous(name = "MES")+  #, limit = c(3*10^6,7.3*10^6))+
scale_x_continuous("Sample  fraction  k")+  #, limit = c(85, 105))+
labs(title = "(a) Goldman Sachs")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
vp=define_region(1,1:3))



df <-data.frame(xxc=kk[1:k150], yyc = XMES_H2[1:k150], gamX2[1:k150])  
dff <-data.frame(xxc=kk[1:k150], yyc = XMES_T2[1:k150], gamX2[1:k150]) 
dfff <-data.frame(xxc=kk[1:k150], yyc = QMES_H2[1:k150], gamX2[1:k150]) 
dd <- ggplot(data=dfff, aes(xxc, yyc)) + geom_line(size = 1, linetype=3) 
print(dd+geom_line(size = 1.5, data = dff, colour = "grey", linetype=1)+
geom_line(size = 1, data = df, colour = "black", linetype=2)+
scale_y_continuous(name = "MES")+  #, limit = c(3*10^6,7.3*10^6))+
scale_x_continuous("Sample  fraction  k")+  #, limit = c(85, 105))+
labs(title = "(b) Morgan Stanley")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(2,1:3))


df <-data.frame(xxc=kk[1:k150], yyc = XMES_H3[1:k150], gamX3[1:k150])  
dff <-data.frame(xxc=kk[1:k150], yyc = XMES_T3[1:k150], gamX3[1:k150]) 
dfff <-data.frame(xxc=kk[1:k150], yyc = QMES_H3[1:k150], gamX3[1:k150]) 
dd <- ggplot(data=dfff, aes(xxc, yyc)) + geom_line(size = 1, linetype=3) 
print(dd+geom_line(size = 1.5, data = dff, colour = "grey", linetype=1)+
geom_line(size = 1, data = df, colour = "black", linetype=2)+
scale_y_continuous(name = "MES")+  #, limit = c(3*10^6,7.3*10^6))+
scale_x_continuous("Sample  fraction  k")+  #, limit = c(85, 105))+
labs(title = "(c) T. Rowe Price")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(3,1:3))





















#__________________________________
#
# (10.13) Calculations of the estimates of $\gamma_X$, $\gamma_Y$ and QMES and XMES (Main article):
#     
#__________________________________

##Sample size:
n = length(ytab);   

##order "pn = 1/n":
pn = 1/n;

##grid of values of "k":
kl = 1; 
# ku = 300; # case of daily data
 ku = 150; # case of weekly data
kk = kl:ku; 
nk = length(kk);

##Hill's estimates:
gamY  = cbind(rep(0, nk));  
gamX1 = cbind(rep(0, nk)); 
gamX2 = cbind(rep(0, nk));
gamX3 = cbind(rep(0, nk));

##MES's estimates for X1=xtab1:
QMES_H1  = cbind(rep(0, nk)); 
            LB_QMES_H1  = cbind(rep(0, nk));
            UB_QMES_H1  = cbind(rep(0, nk));
  
XMES_T1  = cbind(rep(0, nk));
            LB_XMES_T1  = cbind(rep(0, nk));
            UB_XMES_T1  = cbind(rep(0, nk));

##MES's estimates for X2=xtab2:
QMES_H2  = cbind(rep(0, nk));  
            LB_QMES_H2  = cbind(rep(0, nk));
            UB_QMES_H2  = cbind(rep(0, nk));

XMES_T2  = cbind(rep(0, nk));
            LB_XMES_T2  = cbind(rep(0, nk));
            UB_XMES_T2  = cbind(rep(0, nk));

##MES's estimates for X3=xtab3:
QMES_H3  = cbind(rep(0, nk));   
            LB_QMES_H3  = cbind(rep(0, nk));
            UB_QMES_H3  = cbind(rep(0, nk));

XMES_T3  = cbind(rep(0, nk));
            LB_XMES_T3  = cbind(rep(0, nk));
            UB_XMES_T3  = cbind(rep(0, nk));


for (i in 1:nk)
        { 
        	#sample fraction: 
        	k=kk[i];
        	
        	#Hill's EVI:
        	gamY[i]  = Hill_inverse(ytab,k);
        	gamX1[i] = Hill_inverse(xtab1,k);
        	gamX2[i] = Hill_inverse(xtab2,k);
        	gamX3[i] = Hill_inverse(xtab3,k);

            #QMES'estimates:
            QMES_H1[i]  = QMESh(xtab1,ytab,pn,k)
                z=1.959964;
                LB_QMES_H1[i]  = QMES_H1[i] - ( z*log(k/(n*pn)) * QMES_H1[i] * gamX1[i] /sqrt(k)  )                
                UB_QMES_H1[i]  = QMES_H1[i] + ( z*log(k/(n*pn)) * QMES_H1[i] * gamX1[i] /sqrt(k)  )
            
            
            QMES_H2[i]  = QMESh(xtab2,ytab,pn,k)
                LB_QMES_H2[i]  = QMES_H2[i] - ( z*log(k/(n*pn)) * QMES_H2[i] * gamX2[i] /sqrt(k)  )
                UB_QMES_H2[i]  = QMES_H2[i] + ( z*log(k/(n*pn)) * QMES_H2[i] * gamX2[i] /sqrt(k)  )


            QMES_H3[i]  = QMESh(xtab3,ytab,pn,k)
                LB_QMES_H3[i]  = QMES_H3[i] - ( z*log(k/(n*pn)) * QMES_H3[i] * gamX3[i] /sqrt(k)  )
                UB_QMES_H3[i]  = QMES_H3[i] + ( z*log(k/(n*pn)) * QMES_H3[i] * gamX3[i] /sqrt(k)  )
            
                       
            
            
            #Estimate $\widehat\tau_n'(\alpha_n)$ of the extreme expectile level $\tau_n'(\alpha_n)$:
            htau = 1 - ((pn) * (gamY[i]/(1-gamY[i])));
            
            #XMES_T'estimates:
            XMES_T1[i]  = XMESt(xtab1,ytab,(1-htau),k)
                LB_XMES_T1[i]  = XMES_T1[i] - ( z*log(k/(n*pn)) * XMES_T1[i] * gamX1[i] /sqrt(k)  )                
                UB_XMES_T1[i]  = XMES_T1[i] + ( z*log(k/(n*pn)) * XMES_T1[i] * gamX1[i] /sqrt(k)  )
            
            
            XMES_T2[i]  = XMESt(xtab2,ytab,(1-htau),k)
                LB_XMES_T2[i]  = XMES_T2[i] - ( z*log(k/(n*pn)) * XMES_T2[i] * gamX2[i] /sqrt(k)  )                
                UB_XMES_T2[i]  = XMES_T2[i] + ( z*log(k/(n*pn)) * XMES_T2[i] * gamX2[i] /sqrt(k)  )
            
            
            XMES_T3[i]  = XMESt(xtab3,ytab,(1-htau),k)
                LB_XMES_T3[i]  = XMES_T3[i] - ( z*log(k/(n*pn)) * XMES_T3[i] * gamX3[i] /sqrt(k)  )                
                UB_XMES_T3[i]  = XMES_T3[i] + ( z*log(k/(n*pn)) * XMES_T3[i] * gamX3[i] /sqrt(k)  )
           
        }
         





#__________________________________
#
# (10.14) Plot of the Hill estimates of $\gamma_Y$ and $\gamma_X$: for k<=150
#      Fig 8a: hill_plot_gammaY_daily_b  
#__________________________________

# k150= which(kk==150)  # case of daily data
 k150= which(kk==75)  # case of weekly data

dev.new()

df_gamma <- data.frame(  kk[1:k150], gamX1[1:k150], gamX2[1:k150], gamX3[1:k150], gamY[1:k150])                                  
names(df_gamma) <- c("Sample_fraction_k", "Goldman Sachs", "Morgan Stanley", "T. Rowe Price", "Market index")

ecm <- melt(df_gamma, id = "Sample_fraction_k")

dd <- qplot(Sample_fraction_k, value, data=ecm, geom="line", colour=variable)+geom_line(size = 1.5, linetype=1)
dd+scale_color_manual(values = c("Goldman Sachs"='red',"Morgan Stanley"='#3399ff',"T. Rowe Price"='yellow',"Market index"='green3')) +
# scale_y_continuous(name = "Hill  Estimates")+                      # case of daily data
scale_y_continuous(name = "Hill  Estimates", limit = c(0.17, 0.63))+    # case of weekly data
# scale_x_continuous("Sample  fraction  k")+          # case of daily data
scale_x_continuous("Sample  fraction  k", limit = c(2, 75))    # case of weekly data
labs(title = "(a) Hill Plots")

            ######  black & white version :

 dev.new()
 
 dfX1 <-data.frame(xxc=kk[1:k150], yyc = gamX1[1:k150])  
 dfX2 <-data.frame(xxc=kk[1:k150], yyc = gamX2[1:k150]) 
 dfX3 <-data.frame(xxc=kk[1:k150], yyc = gamX3[1:k150])
 dfY <-data.frame(xxc=kk[1:k150], yyc = gamY[1:k150]) 
 #
 dd <- ggplot(data=dfX1, aes(xxc, yyc)) + geom_line(size = 1.25, colour = "grey", linetype=1) 
 dd+geom_line(size = 1, data = dfX2, colour = "black", linetype=4)+
#geom_line(size = .55, data = dfX2, colour = "black", linetype=4)+
#
geom_line(size = 1.25, data = dfX3, colour = "grey", linetype=3)+
geom_line(size = .75, data = dfX3, colour = "black", linetype=3)+
#
geom_line(size = 1, data = dfY, colour = "black", linetype=2)+
geom_line(size = .75, data = dfY, colour = "black", linetype=2)+
#
 #scale_y_continuous(name = "Estimates", limit = c(0.1, 0.53))+     # case of daily data
 scale_y_continuous(name = "Estimates", limit = c(0.17, 0.63))+    # case of weekly data
 #scale_x_continuous("Sample  fraction  k")+          # case of daily data
 scale_x_continuous("Sample  fraction  k", limit = c(2, 75))+    # case of weekly data
 labs(title = "(a) Hill Plots")+ 
 theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
 panel.grid.minor = element_blank(), axis.line = element_line(colour = "black"))





#__________________________________
#
# (10.15) Averaged estimate $\widehat\gamma_{Y}=??$:
#      
#__________________________________

#The first stable region appears for k\in [k_l, k_u]: 

# k_l= 70; k_u= 100; # case of daily data

 k_l= 25; k_u= 35; # case of weekly data

ind_l = which(kk==k_l);
ind_u = which(kk==k_u);
#Averaged estimate:
mean(gamY[ind_l:ind_u])







#__________________________________
#
# (10.16) Plot of the MES estimates:
#     Fig 8b (Main article): Goldman_Sachs_daily_b
#     
#__________________________________


dev.new()
dff <-data.frame(xxc=kk[1:k150], yyc = XMES_T1[1:k150], gamX1[1:k150]) 
dfff <-data.frame(xxc=kk[1:k150], yyc = QMES_H1[1:k150], gamX1[1:k150]) 
#
dd <- ggplot(data=dff, aes(xxc, yyc,col=gamX1[1:k150])) + geom_line(size = 1.5) 
dd+geom_line(size = .5, data = dfff, colour = "black", linetype=1)+
#
#scale_colour_gradientn(name="gamma_X", colours=rainbow(7)) + 
scale_colour_gradientn(name=expression(hat(gamma)[X]), colours=rainbow(7)) + 
scale_y_continuous(name = "MES  Estimates")+  
scale_x_continuous("Sample  fraction  k")+ 
labs(title = "(b) Goldman Sachs")

            ######  black & white version :

dev.new()
dff <-data.frame(xxc=kk[1:k150], yyc = XMES_T1[1:k150]) 
dfff <-data.frame(xxc=kk[1:k150], yyc = QMES_H1[1:k150]) 
#
dd <- ggplot(data=dff, aes(xxc, yyc)) + geom_line(size = 1.6, colour = "grey") 
dd+geom_line(size = 1, data = dfff, colour = "black", linetype=2)+
#
scale_y_continuous(name = "MES")+  
scale_x_continuous("Sample  fraction  k")+  
labs(title = "(b) Goldman Sachs")+ 
 theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
 panel.grid.minor = element_blank(), axis.line = element_line(colour = "black"))












#__________________________________
#
# (10.17) Averaged MES estimates for Goldman Sachs:
#      
#__________________________________

#The first stable region appears for k\in [k_l, k_u]: 

# k_l=80;  k_u=105; # case of daily data 

 k_l= 27; k_u= 36; # case of weekly data


ind_l = which(kk==k_l);
ind_u = which(kk==k_u);
#Averaged estimate:
c(  mean(XMES_T1[ind_l:ind_u])  ,  mean(QMES_H1[ind_l:ind_u]) )

#  0.3123    0.3077   # case of daily data 
#  0.3423    0.3375   # case of weekly data

# CI:
c(  mean(LB_XMES_T1[ind_l:ind_u])  ,  mean(UB_XMES_T1[ind_l:ind_u])  )
c(  mean(LB_QMES_H1[ind_l:ind_u])  ,  mean(UB_QMES_H1[ind_l:ind_u])  )

#__________________________________
#
# (10.18) Plot of the MES estimates:
#      Fig 8c (Main article): Morgan_Stanley_daily_b 
#     
#__________________________________

dev.new()
dff <-data.frame(xxc=kk[1:k150], yyc = XMES_T2[1:k150], gamX2[1:k150]) 
dfff <-data.frame(xxc=kk[1:k150], yyc = QMES_H2[1:k150], gamX2[1:k150]) 
#
dd <- ggplot(data=dff, aes(xxc, yyc,col=gamX2[1:k150])) + geom_line(size = 1.5) 
dd+geom_line(size = .5, data = dfff, colour = "black", linetype=1)+
#
#scale_colour_gradientn(name="gamma_X", colours=rainbow(7)) + 
scale_colour_gradientn(name=expression(hat(gamma)[X]), colours=rainbow(7)) + 
scale_y_continuous(name = "MES  Estimates")+  
scale_x_continuous("Sample  fraction  k")+  
labs(title = "(c) Morgan Stanley")

            ######  black & white version :

dev.new()
dff <-data.frame(xxc=kk[1:k150], yyc = XMES_T2[1:k150]) 
dfff <-data.frame(xxc=kk[1:k150], yyc = QMES_H2[1:k150]) 
#
dd <- ggplot(data=dff, aes(xxc, yyc)) + geom_line(size = 1.6, colour = "grey") 
dd+geom_line(size = 1, data = dfff, colour = "black", linetype=2)+
#
scale_y_continuous(name = "MES")+  
scale_x_continuous("Sample  fraction  k")+  
labs(title = "(c) Morgan Stanley")+ 
 theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
 panel.grid.minor = element_blank(), axis.line = element_line(colour = "black"))








#__________________________________
#
# (10.19) Averaged MES estimates for Morgan Stanley:
#      
#__________________________________

#The first stable region appears for k\in [k_l, k_u]: 

# k_l=90;  k_u=140;    # case of daily data 

 k_l= 23; k_u= 33; # case of weekly data

ind_l = which(kk==k_l);
ind_u = which(kk==k_u);
#Averaged estimate:
c(  mean(XMES_T2[ind_l:ind_u])  ,  mean(QMES_H2[ind_l:ind_u]) )

#  0.5622    0.5552  # case of daily data 
#  0.6495    0.6641  # case of weekly data

# CI:
c(  mean(LB_XMES_T2[ind_l:ind_u])  ,  mean(UB_XMES_T2[ind_l:ind_u])  )
c(  mean(LB_QMES_H2[ind_l:ind_u])  ,  mean(UB_QMES_H2[ind_l:ind_u])  )




#__________________________________
#
# (10.20) Plot of the MES estimates:
#      Fig 8d (Main article): TRowe_Price_daily_b 
#     
#__________________________________

dev.new()
dff <-data.frame(xxc=kk[1:k150], yyc = XMES_T3[1:k150], gamX3[1:k150]) 
dfff <-data.frame(xxc=kk[1:k150], yyc = QMES_H3[1:k150], gamX3[1:k150]) 
#
dd <- ggplot(data=dff, aes(xxc, yyc,col=gamX3[1:k150])) + geom_line(size = 1.5) 
dd+geom_line(size = .5, data = dfff, colour = "black", linetype=1)+
#
#scale_colour_gradientn(name="gamma_X", colours=rainbow(7)) + 
scale_colour_gradientn(name=expression(hat(gamma)[X]), colours=rainbow(7)) + 
scale_y_continuous(name = "MES  Estimates")+  
scale_x_continuous("Sample  fraction  k")+  
labs(title = "(d) T. Rowe Price")

            ######  black & white version :

dev.new()
dff <-data.frame(xxc=kk[1:k150], yyc = XMES_T3[1:k150]) 
dfff <-data.frame(xxc=kk[1:k150], yyc = QMES_H3[1:k150]) 
#
dd <- ggplot(data=dff, aes(xxc, yyc)) + geom_line(size = 1.6, colour = "grey") 
dd+geom_line(size = 1, data = dfff, colour = "black", linetype=2)+
#
scale_y_continuous(name = "MES")+  
scale_x_continuous("Sample  fraction  k")+  
labs(title = "(d) T. Rowe Price")+ 
 theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
 panel.grid.minor = element_blank(), axis.line = element_line(colour = "black"))




#__________________________________
#
# (10.21) Averaged MES estimates for T. Rowe Price:
#      
#__________________________________

#The first stable region appears for k\in [k_l, k_u]: 

# k_l=75;  k_u=100;    # case of daily data 

k_l= 25; k_u= 33; # case of weekly data

ind_l = which(kk==k_l);
ind_u = which(kk==k_u);
#Averaged estimate:
c(  mean(XMES_T3[ind_l:ind_u])  ,  mean(QMES_H3[ind_l:ind_u]) )

# 0.3308   0.3098    # case of daily data
# 0.3407   0.3405    # case of weekly data

# CI:
c(  mean(LB_XMES_T3[ind_l:ind_u])  ,  mean(UB_XMES_T3[ind_l:ind_u])  )
c(  mean(LB_QMES_H3[ind_l:ind_u])  ,  mean(UB_QMES_H3[ind_l:ind_u])  )













#__________________________________
#
# (10.22) Figure 8 in the main article
#         & Figure 16 in the supplement
#      
#__________________________________
#######################  Hill & the 3 banks in a same figure : ###################################    
#    Figure  " MES_daily_bn "
###########################################################################################
   
# k150= which(kk==150)  # case of daily data
 k150= which(kk==75)  # case of weekly data


dev.new()

#require(grid)
grid.newpage()
pushViewport(viewport(layout = grid.layout(4,4)))

define_region = function(row, col){
	viewport(layout.pos.row=row, layout.pos.col=col)
}


 dfX1 <-data.frame(xxc=kk[1:k150], yyc = gamX1[1:k150])  
 dfX2 <-data.frame(xxc=kk[1:k150], yyc = gamX2[1:k150]) 
 dfX3 <-data.frame(xxc=kk[1:k150], yyc = gamX3[1:k150])
 dfY <-data.frame(xxc=kk[1:k150], yyc = gamY[1:k150]) 
 dd <- ggplot(data=dfX1, aes(xxc, yyc)) + geom_line(size = 1.25, colour = "grey", linetype=1) 
 print(dd+geom_line(size = 1, data = dfX2, colour = "black", linetype=4)+
#geom_line(size = .55, data = dfX2, colour = "black", linetype=4)+
geom_line(size = 1.25, data = dfX3, colour = "grey", linetype=3)+
geom_line(size = .75, data = dfX3, colour = "black", linetype=3)+
geom_line(size = 1, data = dfY, colour = "black", linetype=2)+
geom_line(size = .75, data = dfY, colour = "black", linetype=2)+
 scale_y_continuous(name = "Tail  Index", limit = c(0.1, 0.53))+     # case of daily data
 #scale_y_continuous(name = "Tail  Index", limit = c(0.17, 0.63))+    # case of weekly data
 scale_x_continuous("Sample  fraction  k")+          # case of daily data
 #scale_x_continuous("Sample  fraction  k", limit = c(2, 75))+    # case of weekly data
 labs(title = "(a) Hill Plots")+ 
 theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
 panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(1,1:4))


dff <-data.frame(xxc=kk[1:k150], yyc = XMES_T1[1:k150]) 
dfff <-data.frame(xxc=kk[1:k150], yyc = QMES_H1[1:k150]) 
dd <- ggplot(data=dff, aes(xxc, yyc)) + geom_line(size = 1.6, colour = "grey") 
print(dd+geom_line(size = 1, data = dfff, colour = "black", linetype=2)+
scale_y_continuous(name = "MES")+  
scale_x_continuous("Sample  fraction  k")+  
labs(title = "(b) Goldman Sachs")+ 
 theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
 panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(2,1:4))


dff <-data.frame(xxc=kk[1:k150], yyc = XMES_T2[1:k150]) 
dfff <-data.frame(xxc=kk[1:k150], yyc = QMES_H2[1:k150]) 
dd <- ggplot(data=dff, aes(xxc, yyc)) + geom_line(size = 1.6, colour = "grey") 
print(dd+geom_line(size = 1, data = dfff, colour = "black", linetype=2)+
scale_y_continuous(name = "MES")+  
scale_x_continuous("Sample  fraction  k")+  
labs(title = "(c) Morgan Stanley")+ 
 theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
 panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(3,1:4))
  

dff <-data.frame(xxc=kk[1:k150], yyc = XMES_T3[1:k150]) 
dfff <-data.frame(xxc=kk[1:k150], yyc = QMES_H3[1:k150]) 
dd <- ggplot(data=dff, aes(xxc, yyc)) + geom_line(size = 1.6, colour = "grey") 
print(dd+geom_line(size = 1, data = dfff, colour = "black", linetype=2)+
scale_y_continuous(name = "MES")+  
scale_x_continuous("Sample  fraction  k")+  
labs(title = "(d) T. Rowe Price")+ 
 theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
 panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(4,1:4))





###############  Hill & the 3 banks in a same figure (as before in coulors): #############    
#    Figure " MES_daily "
##########################################################################################
   

 k150= which(kk==150)  # case of daily data
# k150= which(kk==75)  # case of weekly data


dev.new()

#require(grid)
grid.newpage()
pushViewport(viewport(layout = grid.layout(4,4)))

define_region = function(row, col){
	viewport(layout.pos.row=row, layout.pos.col=col)
}


df_gamma <- data.frame(  kk[1:k150], gamX1[1:k150], gamX2[1:k150], gamX3[1:k150], gamY[1:k150])                                  
names(df_gamma) <- c("Sample_fraction_k", "Goldman Sachs", "Morgan Stanley", "T. Rowe Price", "Market index")
ecm <- melt(df_gamma, id = "Sample_fraction_k")
dd <- qplot(Sample_fraction_k, value, data=ecm, geom="line", colour=variable)+geom_line(size = 1.5, linetype=1)
print(dd+scale_color_manual(values = c("Goldman Sachs"='red',"Morgan Stanley"='#3399ff',"T. Rowe Price"='yellow',"Market index"='green3')) +
 scale_y_continuous(name = "Tail  Index")+                      # case of daily data
#scale_y_continuous(name = "Tail  Index", limit = c(0.17, 0.63))+    # case of weekly data
 scale_x_continuous("Sample  fraction  k")+          # case of daily data
#scale_x_continuous("Sample  fraction  k", limit = c(2, 75))+    # case of weekly data
labs(title = "(a) Hill Plots"),
  vp=define_region(1,1:4))


dff <-data.frame(xxc=kk[1:k150], yyc = XMES_T1[1:k150], gamX1[1:k150]) 
dfff <-data.frame(xxc=kk[1:k150], yyc = QMES_H1[1:k150], gamX1[1:k150]) 
dd <- ggplot(data=dff, aes(xxc, yyc,col=gamX1[1:k150])) + geom_line(size = 1.5) 
print(dd+geom_line(size = 1, data = dfff, colour = "black", linetype=1)+
#scale_colour_gradientn(name="gamma_X", colours=rainbow(7)) + 
scale_colour_gradientn(name=expression(hat(gamma)[X]), colours=rainbow(7)) + 
scale_y_continuous(name = "MES")+  
scale_x_continuous("Sample  fraction  k")+ 
labs(title = "(b) Goldman Sachs"),       
  vp=define_region(2,1:4))
  
 
dff <-data.frame(xxc=kk[1:k150], yyc = XMES_T2[1:k150], gamX2[1:k150]) 
dfff <-data.frame(xxc=kk[1:k150], yyc = QMES_H2[1:k150], gamX2[1:k150]) 
dd <- ggplot(data=dff, aes(xxc, yyc,col=gamX2[1:k150])) + geom_line(size = 1.5) 
print(dd+geom_line(size = 1, data = dfff, colour = "black", linetype=1)+
#scale_colour_gradientn(name="gamma_X", colours=rainbow(7)) + 
scale_colour_gradientn(name=expression(hat(gamma)[X]), colours=rainbow(7)) + 
scale_y_continuous(name = "MES")+  
scale_x_continuous("Sample  fraction  k")+  
labs(title = "(c) Morgan Stanley"),       
  vp=define_region(3,1:4))
  

dff <-data.frame(xxc=kk[1:k150], yyc = XMES_T3[1:k150], gamX3[1:k150]) 
dfff <-data.frame(xxc=kk[1:k150], yyc = QMES_H3[1:k150], gamX3[1:k150]) 
dd <- ggplot(data=dff, aes(xxc, yyc,col=gamX3[1:k150])) + geom_line(size = 1.5) 
print(dd+geom_line(size = 1, data = dfff, colour = "black", linetype=1)+
#scale_colour_gradientn(name="gamma_X", colours=rainbow(7)) + 
scale_colour_gradientn(name=expression(hat(gamma)[X]), colours=rainbow(7)) + 
scale_y_continuous(name = "MES")+  
scale_x_continuous("Sample  fraction  k")+  
labs(title = "(d) T. Rowe Price"),       
  vp=define_region(4,1:4))







#__________________________________
#
# (10.23)  Confidence intervals :
#      
#__________________________________
#######################  Confidence intervals : ###################################    
#    Figure  " MES_daily_bn_CI " +  " MES_weekly_bn_CI "
###########################################################################################
   
   
# k150= which(kk==150)  # case of daily data
 k150= which(kk==75)  # case of weekly data


dev.new()

#require(grid)
grid.newpage()
pushViewport(viewport(layout = grid.layout(3,2)))

define_region = function(row, col){
	viewport(layout.pos.row=row, layout.pos.col=col)
}


df <-data.frame(xxc=kk[1:k150], yyc = XMES_T1[1:k150]) 
df_LB <-data.frame(xxc=kk[1:k150], yyc = LB_XMES_T1[1:k150])
df_UB <-data.frame(xxc=kk[1:k150], yyc = UB_XMES_T1[1:k150]) 

dd <- ggplot(data=df, aes(xxc, yyc)) + geom_line(size = 1, colour = "black") 
print(dd+geom_line(size = 1, data = df_LB, colour = "grey", linetype=2)+
geom_line(size = 1, data = df_UB, colour = "grey", linetype=2)+
scale_y_continuous(name = "MES")+  
scale_x_continuous("k")+  
labs(title = "Goldman Sachs: XMES")+ 
 theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
 panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(1,1))
    
   
df <-data.frame(xxc=kk[1:k150], yyc = QMES_H1[1:k150]) 
df_LB <-data.frame(xxc=kk[1:k150], yyc = LB_QMES_H1[1:k150])
df_UB <-data.frame(xxc=kk[1:k150], yyc = UB_QMES_H1[1:k150]) 

dd <- ggplot(data=df, aes(xxc, yyc)) + geom_line(size = 1, colour = "black") 
print(dd+geom_line(size = 1, data = df_LB, colour = "grey", linetype=2)+
geom_line(size = 1, data = df_UB, colour = "grey", linetype=2)+
scale_y_continuous(name = "MES")+  
scale_x_continuous("k")+  
labs(title = "Goldman Sachs: QMES")+ 
 theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
 panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(1,2))
  
 
df <-data.frame(xxc=kk[1:k150], yyc = XMES_T2[1:k150]) 
df_LB <-data.frame(xxc=kk[1:k150], yyc = LB_XMES_T2[1:k150])
df_UB <-data.frame(xxc=kk[1:k150], yyc = UB_XMES_T2[1:k150])

dd <- ggplot(data=df, aes(xxc, yyc)) + geom_line(size = 1, colour = "black") 
print(dd+geom_line(size = 1, data = df_LB, colour = "grey", linetype=2)+
geom_line(size = 1, data = df_UB, colour = "grey", linetype=2)+
scale_y_continuous(name = "MES")+  
scale_x_continuous("k")+  
labs(title = "Morgan Stanley: XMES")+ 
 theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
 panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(2,1))
    
   
df <-data.frame(xxc=kk[1:k150], yyc = QMES_H2[1:k150]) 
df_LB <-data.frame(xxc=kk[1:k150], yyc = LB_QMES_H2[1:k150])
df_UB <-data.frame(xxc=kk[1:k150], yyc = UB_QMES_H2[1:k150]) 

dd <- ggplot(data=df, aes(xxc, yyc)) + geom_line(size = 1, colour = "black") 
print(dd+geom_line(size = 1, data = df_LB, colour = "grey", linetype=2)+
geom_line(size = 1, data = df_UB, colour = "grey", linetype=2)+
scale_y_continuous(name = "MES")+  
scale_x_continuous("k")+  
labs(title = "Morgan Stanley: QMES")+ 
 theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
 panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(2,2))
  
 

df <-data.frame(xxc=kk[1:k150], yyc = XMES_T3[1:k150]) 
df_LB <-data.frame(xxc=kk[1:k150], yyc = LB_XMES_T3[1:k150])
df_UB <-data.frame(xxc=kk[1:k150], yyc = UB_XMES_T3[1:k150])

dd <- ggplot(data=df, aes(xxc, yyc)) + geom_line(size = 1, colour = "black") 
print(dd+geom_line(size = 1, data = df_LB, colour = "grey", linetype=2)+
geom_line(size = 1, data = df_UB, colour = "grey", linetype=2)+
scale_y_continuous(name = "MES")+  
scale_x_continuous("k")+  
labs(title = "T. Rowe Price: XMES")+ 
 theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
 panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(3,1))
    
   
df <-data.frame(xxc=kk[1:k150], yyc = QMES_H3[1:k150]) 
df_LB <-data.frame(xxc=kk[1:k150], yyc = LB_QMES_H3[1:k150])
df_UB <-data.frame(xxc=kk[1:k150], yyc = UB_QMES_H3[1:k150]) 

dd <- ggplot(data=df, aes(xxc, yyc)) + geom_line(size = 1, colour = "black") 
print(dd+geom_line(size = 1, data = df_LB, colour = "grey", linetype=2)+
geom_line(size = 1, data = df_UB, colour = "grey", linetype=2)+
scale_y_continuous(name = "MES")+  
scale_x_continuous("k")+  
labs(title = "T. Rowe Price: QMES")+ 
 theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
 panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(3,2))
  

